@@ -22,14 +22,13 @@ through vectorization (`Numpy <https://www.numpy.org/>`_), parallelization
 (`mpi4py <https://bitbucket.org/mpi4py/mpi4py>`_) and by moving critical
 routines to `Cython <https://cython.org/>`_.
 
-``Shenfun`` has some support for solving scalar equations in curvilinear
-coordinates, like polar or spherical coordinates. It is (yet) not
-possible to solve vector-equations using curvilinear coordinates.
+``Shenfun`` has support for solving scalar and vector equations in curvilinear
+coordinates, like polar or spherical coordinates.
 
 """
 #pylint: disable=wildcard-import,no-name-in-module
 
-__version__ = '2.2.2'
+__version__ = '3.0.6'
 __author__ = 'Mikael Mortensen'
 
 import numpy as np
@@ -45,7 +44,7 @@ from . import la
 from .fourier import energy_fourier
 from .io import *
 from .matrixbase import *
-from .spectralbase import inner_product
+from .spectralbase import inner_product, MixedFunctionSpace
 from .forms import *
 from .tensorproductspace import *
 from .utilities import *
