@@ -5,10 +5,9 @@ import numpy as np
 from shenfun.optimization import optimizer
 from shenfun.optimization.cython import la
 from shenfun.la import TDMA as la_TDMA
-from shenfun.utilities import inheritdocstrings
 from shenfun.matrixbase import TPMatrix
 
-@inheritdocstrings
+
 class TDMA(la_TDMA):
 
     def __call__(self, b, u=None, axis=0):
@@ -28,7 +27,8 @@ class TDMA(la_TDMA):
             u /= self.mat.scale
         return u
 
-class Helmholtz(object):
+
+class Helmholtz:
     r"""Helmholtz solver
 
     .. math::
@@ -241,7 +241,8 @@ class Helmholtz(object):
     def Helmholtz_matvec(v, b, alfa, beta, dd, ud, bd, axis=0):
         raise NotImplementedError("Use Cython or Numba")
 
-class Biharmonic(object):
+
+class Biharmonic:
     r"""Multidimensional Biharmonic solver for
 
     .. math::
@@ -440,7 +441,7 @@ class Biharmonic(object):
         return c
 
 
-class PDMA(object):
+class PDMA:
     r"""Pentadiagonal matrix solver
 
     Pentadiagonal matrix with diagonals in offsets -4, -2, 0, 2, 4
