@@ -30,7 +30,7 @@ to be applied to each. As such the mass matrix BDDmat may be looked up
 as
 
 >>> from shenfun.chebyshev.matrices import mat
->>> from shenfun.chebyshev.bases import ShenDirichletBasis as SD
+>>> from shenfun.chebyshev.bases import ShenDirichlet as SD
 >>> B = mat[((SD, 0), (SD, 0))]
 
 and an instance of the matrix can be created as
@@ -69,18 +69,17 @@ import functools
 import numpy as np
 from shenfun.optimization import cython
 from shenfun.matrixbase import SpectralMatrix
-from shenfun.utilities import inheritdocstrings
 from shenfun.la import TDMA as neumann_TDMA
 from .la import TDMA
 from . import bases
 
 # Short names for instances of bases
-CB = bases.Basis
-SD = bases.ShenDirichletBasis
-SB = bases.ShenBiharmonicBasis
-SN = bases.ShenNeumannBasis
-BD = bases.BCBasis
-BB = bases.BCBiharmonicBasis
+CB = bases.Orthogonal
+SD = bases.ShenDirichlet
+SB = bases.ShenBiharmonic
+SN = bases.ShenNeumann
+BD = bases.BCDirichlet
+BB = bases.BCBiharmonic
 
 def get_ck(N, quad):
     """Return array ck, parameter in Chebyshev expansions
@@ -105,7 +104,7 @@ def dmax(N, M, d):
     Z = min(N, M)
     return Z-abs(d)+min(max((M-N)*int(d/abs(d)), 0), abs(d))
 
-@inheritdocstrings
+
 class BDDmat(SpectralMatrix):
     r"""Matrix for inner product
 
@@ -131,6 +130,7 @@ class BDDmat(SpectralMatrix):
         d[-2] = d[2]
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
         self.solve = TDMA(self)
+        self._matvec_methods += ['cython', 'self']
 
     def matvec(self, v, c, format='cython', axis=0):
         N, M = self.shape
@@ -171,7 +171,6 @@ class BDDmat(SpectralMatrix):
         return c
 
 
-@inheritdocstrings
 class BNDmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -213,7 +212,6 @@ class BNDmat(SpectralMatrix):
         return c
 
 
-@inheritdocstrings
 class BDNmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -245,6 +243,7 @@ class BDNmat(SpectralMatrix):
              2: -np.pi/2}
         d[-2] = (-np.pi/2*(k/(k+2))**2)[:dmax(N-2, M-2, -2)]
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        self._matvec_methods += ['cython']
 
     def matvec(self, v, c, format='cython', axis=0):
         c.fill(0)
@@ -266,7 +265,6 @@ class BDNmat(SpectralMatrix):
         return c
 
 
-@inheritdocstrings
 class BNTmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -300,7 +298,6 @@ class BNTmat(SpectralMatrix):
         return c
 
 
-@inheritdocstrings
 class BNBmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -334,7 +331,6 @@ class BNBmat(SpectralMatrix):
         return c
 
 
-@inheritdocstrings
 class BTTmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -355,6 +351,7 @@ class BTTmat(SpectralMatrix):
         assert isinstance(trial[0], CB)
         ck = get_ck(min(test[0].N, trial[0].N), test[0].quad)
         SpectralMatrix.__init__(self, {0: np.pi/2*ck}, test, trial)
+        self._matvec_methods += ['self']
 
     def matvec(self, v, c, format='csr', axis=0):
         c.fill(0)
@@ -388,7 +385,6 @@ class BTTmat(SpectralMatrix):
         u[ss] = b[ss]*d[sl]
         return u
 
-@inheritdocstrings
 class BNNmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -432,7 +428,6 @@ class BNNmat(SpectralMatrix):
         return c
 
 
-@inheritdocstrings
 class BDTmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -461,7 +456,6 @@ class BDTmat(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 
-@inheritdocstrings
 class BTDmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -490,7 +484,6 @@ class BTDmat(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 
-@inheritdocstrings
 class BTNmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -518,7 +511,6 @@ class BTNmat(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 
-@inheritdocstrings
 class BBBmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -552,6 +544,7 @@ class BBBmat(SpectralMatrix):
         d[-4] = d4[:dmax(N-4, M-4, -4)]
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
         self.solve = PDMA(self)
+        self._matvec_methods += ['cython', 'self']
 
     def matvec(self, v, c, format='cython', axis=0):
         c.fill(0)
@@ -596,7 +589,6 @@ class BBBmat(SpectralMatrix):
         return c
 
 
-@inheritdocstrings
 class BBDmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -626,6 +618,7 @@ class BBDmat(SpectralMatrix):
               2: -(a+b*ck[4:])*np.pi/2,
               4: b[:-2]*np.pi/2}
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        self._matvec_methods += ['cython', 'self']
 
     def matvec(self, v, c, format='cython', axis=0):
         c.fill(0)
@@ -659,7 +652,7 @@ class BBDmat(SpectralMatrix):
 
         return c
 
-@inheritdocstrings
+
 class CDNmat(SpectralMatrix):
     r"""Matrix for inner product
 
@@ -687,8 +680,10 @@ class CDNmat(SpectralMatrix):
         d = {-1: -((k[1:]-1)/(k[1:]+1))**2*(k[1:]+1)*np.pi,
               1: (k[:-1]+1)*np.pi}
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        self._matvec_methods += ['cython']
 
     def matvec(self, v, c, format='cython', axis=0):
+        c.fill(0)
         if format == 'cython' and v.ndim == 3:
             cython.Matvec.CDN_matvec3D_ptr(v, c, self[-1], self[1], axis)
             self.scale_array(c)
@@ -704,7 +699,6 @@ class CDNmat(SpectralMatrix):
         return c
 
 
-@inheritdocstrings
 class CDDmat(SpectralMatrix):
     r"""Matrix for inner product
 
@@ -728,6 +722,7 @@ class CDDmat(SpectralMatrix):
         d = {-1: -(k[1:N-2]+1)*np.pi,
               1: (k[:(N-3)]+1)*np.pi}
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        self._matvec_methods += ['cython', 'self']
 
     def matvec(self, v, c, format='cython', axis=0):
         N = self.shape[0]
@@ -760,7 +755,6 @@ class CDDmat(SpectralMatrix):
         return c
 
 
-@inheritdocstrings
 class CNDmat(SpectralMatrix):
     r"""Matrix for inner product
 
@@ -799,7 +793,6 @@ class CNDmat(SpectralMatrix):
         return c
 
 
-@inheritdocstrings
 class CTDmat(SpectralMatrix):
     r"""Matrix for inner product
 
@@ -852,8 +845,48 @@ class CDTmat(SpectralMatrix):
         d = {1: np.pi*(k[:N-2]+1)}
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
+class CTTmat(SpectralMatrix):
+    r"""Matrix for inner product
+
+    .. math::
+
+        C_{kj} = (T'_j, T_k)_w
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N \text{ and } k = 0, 1, ..., N
+
+    :math:`T_k` is the Chebyshev basis function.
+    """
+    def __init__(self, test, trial, measure=1):
+        assert isinstance(test[0], CB)
+        assert isinstance(trial[0], CB)
+        N = test[0].N
+        k = np.arange(N, dtype=np.float)
+        d = {}
+        for i in range(1, N, 2):
+            d[i] = np.pi*k[i:]
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        self._matvec_methods += ['cython']
+
+    def matvec(self, v, c, format='cython', axis=0):
+        c.fill(0)
+        if format == 'cython' and v.ndim == 3:
+            cython.Matvec.CTT_matvec3D_ptr(v, c, axis)
+            self.scale_array(c)
+        elif format == 'cython' and v.ndim == 2:
+            cython.Matvec.CTT_matvec2D_ptr(v, c, axis)
+            self.scale_array(c)
+        elif format == 'cython' and v.ndim == 1:
+            cython.Matvec.CTT_matvec(v, c)
+            self.scale_array(c)
+        else:
+            c = super(CTTmat, self).matvec(v, c, format=format, axis=axis)
+        return c
+
 
-@inheritdocstrings
 class CBDmat(SpectralMatrix):
     r"""Matrix for inner product
 
@@ -879,6 +912,7 @@ class CBDmat(SpectralMatrix):
               1: 2*(k[:N-4]+1)*np.pi,
               3: -(k[:N-5]+1)*np.pi}
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        self._matvec_methods += ['cython', 'self']
 
     def matvec(self, v, c, format='cython', axis=0):
         N, M = self.shape
@@ -909,7 +943,6 @@ class CBDmat(SpectralMatrix):
         return c
 
 
-@inheritdocstrings
 class CDBmat(SpectralMatrix):
     r"""Matrix for inner product
 
@@ -933,6 +966,7 @@ class CDBmat(SpectralMatrix):
              -1: -2*(k[1:-3]+1)**2/(k[1:-3]+2)*np.pi,
               1: (k[:-5]+1)*np.pi}
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        self._matvec_methods += ['cython', 'self']
 
     def matvec(self, v, c, format='cython', axis=0):
         N, M = self.shape
@@ -966,7 +1000,6 @@ class CDBmat(SpectralMatrix):
         return c
 
 
-@inheritdocstrings
 class ABBmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
 
@@ -992,6 +1025,7 @@ class ABBmat(SpectralMatrix):
               0: -4*((k+1)*(k+2)**2)/(k+3)*np.pi,
               2: 2*(k[:-2]+1)*(k[:-2]+2)*np.pi}
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        self._matvec_methods += ['cython', 'self']
 
     def matvec(self, v, c, format='cython', axis=0):
         N = self.shape[0]
@@ -1026,7 +1060,6 @@ class ABBmat(SpectralMatrix):
         return c
 
 
-@inheritdocstrings
 class ADDmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
 
@@ -1052,6 +1085,7 @@ class ADDmat(SpectralMatrix):
         for i in range(2, N-2, 2):
             d[i] = -4*np.pi*(k[:-(i+2)]+1)
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        self._matvec_methods += ['cython']
 
         # Following storage more efficient, but requires effort in iadd/isub...
 #        d = {0: -2*np.pi*(k[:N-2]+1)*(k[:N-2]+2),
@@ -1122,7 +1156,6 @@ class ADDmat(SpectralMatrix):
         return u
 
 
-@inheritdocstrings
 class ANNmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
 
@@ -1210,7 +1243,6 @@ class ANNmat(SpectralMatrix):
         return u
 
 
-@inheritdocstrings
 class ATTmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
 
@@ -1236,9 +1268,25 @@ class ATTmat(SpectralMatrix):
         for j in range(2, N, 2):
             d[j] = k[j:]*(k[j:]**2-k[:-j]**2)*np.pi/2.
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        self._matvec_methods += ['cython']
+
+    def matvec(self, v, c, format='cython', axis=0):
+        c.fill(0)
+        if format == 'cython' and v.ndim == 3:
+            cython.Matvec.ATT_matvec3D_ptr(v, c, axis)
+            self.scale_array(c)
+        elif format == 'cython' and v.ndim == 2:
+            cython.Matvec.ATT_matvec2D_ptr(v, c, axis)
+            self.scale_array(c)
+        elif format == 'cython' and v.ndim == 1:
+            cython.Matvec.ATT_matvec(v, c)
+            self.scale_array(c)
+        else:
+            c = super(ATTmat, self).matvec(v, c, format=format, axis=axis)
+        return c
+
 
 
-@inheritdocstrings
 class SBBmat(SpectralMatrix):
     r"""Biharmonic matrix for inner product
 
@@ -1266,6 +1314,7 @@ class SBBmat(SpectralMatrix):
             i = 8*(ki[:-j]+1)*(ki[:-j]+2)*(ki[:-j]*(ki[:-j]+4)+3*(ki[j:]+2)**2)
             d[j] = np.array(i*np.pi/(k[j:]+3))
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        self._matvec_methods += ['cython']
 
     def matvec(self, v, c, format='cython', axis=0):
         c.fill(0)
@@ -1284,7 +1333,7 @@ class SBBmat(SpectralMatrix):
 
         return c
 
-@inheritdocstrings
+
 class BCDmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -1311,7 +1360,7 @@ class BCDmat(SpectralMatrix):
 
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
-@inheritdocstrings
+
 class BCBmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -1332,8 +1381,6 @@ class BCBmat(SpectralMatrix):
     def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], BB)
-        N = test[0].N
-        k = np.arange(N-4, dtype=np.float)
         d = {0: np.array([np.pi/2, np.pi*21/64, -np.pi/16, np.pi/32]),
              1: np.array([np.pi/2, -np.pi*5/64, np.pi/16]),
              2: np.array([np.pi*5/24, -np.pi*5/64]),
@@ -1341,11 +1388,9 @@ class BCBmat(SpectralMatrix):
             -1: np.array([-np.pi*21/64, 0, np.pi/32, 0]),
             -2: np.array([0, -np.pi/32, 0, 0]),
             -3: np.array([np.pi/32, 0, 0, 0])}
-
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 
-@inheritdocstrings
 class _Chebmatrix(SpectralMatrix):
     def __init__(self, test, trial, measure=1):
         SpectralMatrix.__init__(self, {}, test, trial, measure=measure)
@@ -1394,6 +1439,7 @@ mat = _ChebMatDict({
     ((SD, 0), (SN, 1)): CDNmat,
     ((SB, 0), (SD, 1)): CBDmat,
     ((CB, 0), (SD, 1)): CTDmat,
+    ((CB, 0), (CB, 1)): CTTmat,
     ((SD, 0), (SD, 1)): CDDmat,
     ((SN, 0), (SD, 1)): CNDmat,
     ((SD, 0), (SB, 1)): CDBmat,
