@@ -4,9 +4,10 @@ weighted inner product.
 """
 from numbers import Number
 import numpy as np
-from shenfun.spectralbase import inner_product, SpectralBase, MixedBasis
-from shenfun.matrixbase import TPMatrix
-from shenfun.tensorproductspace import TensorProductSpace, MixedTensorProductSpace
+import sympy as sp
+from shenfun.spectralbase import inner_product, SpectralBase, MixedFunctionSpace
+from shenfun.matrixbase import TPMatrix, Identity
+from shenfun.tensorproductspace import TensorProductSpace, CompositeSpace
 from shenfun.utilities import dx, split
 from .arguments import Expr, Function, BasisFunction, Array
 
@@ -17,19 +18,24 @@ __all__ = ('inner',)
 
 def inner(expr0, expr1, output_array=None, level=0):
     r"""
-    Return (weighted) discrete inner product of linear or bilinear form
+    Return (weighted or unweighted) discrete inner product of kind
 
     .. math::
 
-        (f, g)_w^N = \sum_{i\in\mathcal{I}}f(x_i) \overline{g}(x_i) w_i \approx \int_{\Omega} g\, \overline{f}\, w\, dx
+        (f, g)_w^N = \sum_{i\in\mathcal{I}}f(x_i) \overline{g}(x_i) w_i \approx \int_{\Omega} f\, \overline{g}\, w\, dx
 
     where :math:`\mathcal{I}=0, 1, \ldots, N, N \in \mathbb{Z}^+`, :math:`f`
-    is an expression linear in a :class:`.TestFunction`, and :math:`g` is an
-    expression that is linear in :class:`.TrialFunction` or :class:`.Function`,
-    or it is simply an :class:`.Array` (a solution interpolated on the
+    is a number or an expression linear in a :class:`.TestFunction`,
+    and :math:`g` is an expression that is linear in :class:`.TrialFunction`
+    or :class:`.Function`, or it is simply an :class:`.Array` (a solution interpolated on the
     quadrature mesh in physical space). :math:`w` is a weight associated with
     chosen basis, and :math:`w_i` are quadrature weights.
 
+    Note
+    ----
+        If :math:`f` is a number (typically one) and :math:`g` an :class:`.Array`, then `inner`
+        represents an unweighted, regular integral over the domain.
+
     If the expressions are created in a multidimensional :class:`.TensorProductSpace`,
     then the sum above is over all dimensions. In 2D it becomes:
 
@@ -41,8 +47,7 @@ def inner(expr0, expr1, output_array=None, level=0):
 
     Parameters
     ----------
-    expr0, expr1 : :class:`.Expr`, :class:`.BasisFunction`, :class:`.Array`
-        or number.
+    expr0, expr1 : :class:`.Expr`, :class:`.BasisFunction`, :class:`.Array`, number
         Either one can be an expression involving a
         BasisFunction (:class:`.TestFunction`, :class:`.TrialFunction` or
         :class:`.Function`) an Array or a number. With expressions (Expr) on a
@@ -97,9 +102,9 @@ def inner(expr0, expr1, output_array=None, level=0):
     -------
     Compute mass matrix of Shen's Chebyshev Dirichlet basis:
 
-    >>> from shenfun import Basis
-    >>> from shenfun import TestFunction, TrialFunction
-    >>> SD = Basis(6, 'Chebyshev', bc=(0, 0))
+    >>> from shenfun import FunctionSpace, TensorProductSpace
+    >>> from shenfun import TestFunction, TrialFunction, Array
+    >>> SD = FunctionSpace(6, 'Chebyshev', bc=(0, 0))
     >>> u = TrialFunction(SD)
     >>> v = TestFunction(SD)
     >>> B = inner(v, u)
@@ -109,6 +114,13 @@ def inner(expr0, expr1, output_array=None, level=0):
     >>> [np.all(abs(B[k]-v) < 1e-7) for k, v in d.items()]
     [True, True, True]
 
+    # Compute unweighted integral
+    >>> F = FunctionSpace(10, 'F', domain=(0, 2*np.pi))
+    >>> T = TensorProductSpace(comm, (SD, F))
+    >>> area = inner(1, Array(T, val=1))
+    >>> print('Area of domain =', area)
+    Area of domain = 12.56637061435917
+
     """
     # Wrap a pure numpy array in Array
     if isinstance(expr0, np.ndarray) and not isinstance(expr0, (Array, Function)):
@@ -125,12 +137,13 @@ def inner(expr0, expr1, output_array=None, level=0):
     if isinstance(expr0, Number):
         assert isinstance(expr1, (Array, Function))
         space = expr1.function_space()
-        if isinstance(space, (TensorProductSpace, MixedTensorProductSpace)):
+        if isinstance(space, (TensorProductSpace, CompositeSpace)):
             df = np.prod(np.array([base.domain_factor() for base in space.bases]))
         elif isinstance(space, SpectralBase):
             df = space.domain_factor()
         if isinstance(expr1, Function):
-            return (expr0/df)*dx(expr1.backward())
+            #return (expr0/df)*dx(expr1.backward())
+            expr1 = expr1.backward()
         if hasattr(space, 'hi'):
             if space.hi.prod() != 1:
                 expr1 = space.get_measured_array(expr1.copy())
@@ -140,12 +153,13 @@ def inner(expr0, expr1, output_array=None, level=0):
     if isinstance(expr1, Number):
         assert isinstance(expr0, (Array, Function))
         space = expr0.function_space()
-        if isinstance(space, (TensorProductSpace, MixedTensorProductSpace)):
+        if isinstance(space, (TensorProductSpace, CompositeSpace)):
             df = np.prod(np.array([base.domain_factor() for base in space.bases]))
         elif isinstance(space, SpectralBase):
             df = space.domain_factor()
         if isinstance(expr0, Function):
-            return (expr1/df)*dx(expr0.backward())
+            #return (expr1/df)*dx(expr0.backward())
+            expr0 = expr0.backward()
         if hasattr(space, 'hi'):
             if space.hi.prod() != 1:
                 expr0 = space.get_measured_array(expr0.copy())
@@ -155,7 +169,7 @@ def inner(expr0, expr1, output_array=None, level=0):
     if isinstance(expr0, tuple):
         assert isinstance(expr1, (Array, Function))
         space = expr1.function_space()
-        assert isinstance(space, MixedTensorProductSpace)
+        assert isinstance(space, CompositeSpace)
         assert len(expr0) == len(space)
         result = 0.0
         for e0i, e1i in zip(expr0, expr1):
@@ -165,7 +179,7 @@ def inner(expr0, expr1, output_array=None, level=0):
     if isinstance(expr1, tuple):
         assert isinstance(expr0, (Array, Function))
         space = expr0.function_space()
-        assert isinstance(space, MixedTensorProductSpace)
+        assert isinstance(space, CompositeSpace)
         assert len(expr1) == len(space)
         result = 0.0
         for e0i, e1i in zip(expr0, expr1):
@@ -186,14 +200,16 @@ def inner(expr0, expr1, output_array=None, level=0):
     else:
         raise RuntimeError
 
-    if isinstance(test, BasisFunction):
-        recursive = test.rank > 0
-    elif isinstance(test, Expr):
-        recursive = test.function_space().is_composite_space
+    #if isinstance(test, BasisFunction):
+    #    recursive = test.tensor_rank > 0
+    #elif isinstance(test, Expr):
+    recursive = test.function_space().is_composite_space
     if isinstance(trial, Array):
-        assert trial.rank == test.rank
+        assert trial.tensor_rank == test.tensor_rank
     elif isinstance(trial, BasisFunction):
         recursive *= (trial.expr_rank() > 0)
+    if test.expr_rank() == 0:
+        recursive = False
 
     if recursive: # Use recursive algorithm for vector expressions of expr_rank > 0, e.g., inner(v, grad(u))
         if output_array is None and trial.argument == 2:
@@ -202,23 +218,54 @@ def inner(expr0, expr1, output_array=None, level=0):
         gij = test.function_space().coors.get_covariant_metric_tensor()
         if trial.argument == 2:
             # linear form
-            w0 = np.zeros_like(output_array[0])
-            for i, (te, x) in enumerate(zip(test, output_array)):
-                for j, tr in enumerate(trial):
-                    if gij[i, j] == 0:
-                        continue
-                    w0.fill(0)
-                    x += inner(te*gij[i, j], tr, output_array=w0)
+
+            if test.tensor_rank == 2:
+                w0 = np.zeros_like(output_array[0][0])
+                for i, (tei, xi) in enumerate(zip(test, output_array)):
+                    for j, (teij, xij) in enumerate(zip(tei, xi)):
+                        for k, trk in enumerate(trial):
+                            if gij[i, k] == 0:
+                                continue
+                            for l, trkl in enumerate(trk):
+                                if gij[j, l] == 0:
+                                    continue
+
+                                w0.fill(0)
+                                xij += inner(teij*gij[i, k]*gij[j, l], trkl, output_array=w0)
+
+            elif test.tensor_rank == 1:
+                w0 = np.zeros_like(output_array[0])
+                for i, (te, x) in enumerate(zip(test, output_array)):
+                    for j, tr in enumerate(trial):
+                        if gij[i, j] == 0:
+                            continue
+                        w0.fill(0)
+                        x += inner(te*gij[i, j], tr, output_array=w0)
+
             return output_array
 
         result = []
 
-        for i, te in enumerate(test):
-            for j, tr in enumerate(trial):
-                if gij[i, j] == 0:
-                    continue
-                l = inner(te*gij[i, j], tr, level=level)
-                result += l if isinstance(l, list) else [l]
+        if test.tensor_rank == 2:
+            for i, tei in enumerate(test):
+                for j, teij in enumerate(tei):
+                    for k, trk in enumerate(trial):
+                        if gij[i, k] == 0:
+                            continue
+                        for l, trkl in enumerate(trk):
+                            if gij[j, l] == 0:
+                                continue
+                            p = inner(teij*gij[i, k]*gij[j, l], trkl, level=level)
+                            result += p if isinstance(p, list) else [p]
+
+        elif test.tensor_rank == 1:
+            for i, te in enumerate(test):
+                for j, tr in enumerate(trial):
+                    if gij[i, j] == 0:
+                        continue
+                    l = inner(te, tr*gij[i, j], level=level)
+                    result += l if isinstance(l, list) else [l]
+
         return result[0] if len(result) == 1 else result
 
     if output_array is None and trial.argument == 2:
@@ -230,7 +277,7 @@ def inner(expr0, expr1, output_array=None, level=0):
         assert test.argument == 0
         space = test.function_space()
         if isinstance(trial, Array):
-            if trial.rank == 0:
+            if trial.tensor_rank == 0:
                 output_array = space.scalar_product(trial, output_array)
                 return output_array
             trial = trial.forward()
@@ -244,6 +291,7 @@ def inner(expr0, expr1, output_array=None, level=0):
         test = Expr(test)
 
     assert test.expr_rank() == trial.expr_rank()
+
     testspace = test.base.function_space()
     trialspace = trial.base.function_space()
     test_scale = test.scales()
@@ -262,19 +310,19 @@ def inner(expr0, expr1, output_array=None, level=0):
                 continue
             for test_j, b0 in enumerate(base_test):              # second index test
                 for trial_j, b1 in enumerate(base_trial):        # second index trial
-                    dV = test_scale[vec_i][test_j]*trial_scale[vec_j][trial_j]*testspace.hi.prod()*g
+                    dV = sp.simplify(test_scale[vec_i][test_j]*trial_scale[vec_j][trial_j]*testspace.coors.get_sqrt_det_g()*g)
+                    dV = sp.refine(dV, testspace.coors._assumptions)
                     assert len(b0) == len(b1)
                     trial_sp = trialspace
-                    if isinstance(trialspace, (MixedTensorProductSpace, MixedBasis)): # could operate on a vector, e.g., div(u), where u is vector
+                    if isinstance(trialspace, (CompositeSpace, MixedFunctionSpace)): # could operate on a vector, e.g., div(u), where u is vector
                         trial_sp = trialspace.flatten()[trial_ind[trial_j]]
                     test_sp = testspace
-                    if isinstance(testspace, (MixedTensorProductSpace, MixedBasis)):
+                    if isinstance(testspace, (CompositeSpace, MixedFunctionSpace)):
                         test_sp = testspace.flatten()[test_ind[test_j]]
                     has_bcs = False
                     # Check if scale is zero
                     if dV == 0:
                         continue
-
                     for dv in split(dV):
                         sc = dv['coeff']
                         scb = dv['coeff']
@@ -284,10 +332,13 @@ def inner(expr0, expr1, output_array=None, level=0):
                             ts = trial_sp[i]
                             tt = test_sp[i]
                             msx = 'xyzrs'[i]
-                            msi = dv[msx]
+                            msi = sp.simplify(dv[msx])
 
                             # assemble inner product
                             AA = inner_product((tt, a), (ts, b), msi)
+                            if len(AA) == 0:
+                                AA = Identity(AA.shape, scale=0)
+
                             M.append(AA)
                             if not abs(AA.scale-1.) < 1e-8:
                                 sc *= AA.scale
@@ -312,7 +363,7 @@ def inner(expr0, expr1, output_array=None, level=0):
                             M[0].mixedbase = testspace
                             A.append(M[0])
                         else:
-                            A.append(TPMatrix(M, test_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
+                            A.append(TPMatrix(M, test_sp, trial_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
                         if has_bcs:
                             if len(DM) == 1: # 1D case
                                 DM[0].global_index = (test_ind[test_j], trial_ind[trial_j])
@@ -320,7 +371,12 @@ def inner(expr0, expr1, output_array=None, level=0):
                                 DM[0].mixedbase = testspace
                                 A.append(DM[0])
                             else:
-                                A.append(TPMatrix(DM, test_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
+                                if len(trial_sp.get_nonhomogeneous_axes()) == 1:
+                                    A.append(TPMatrix(DM, test_sp, trial_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
+                                elif len(trial_sp.get_nonhomogeneous_axes()) == 2:
+                                    A.append(TPMatrix([M[0], DM[1]], test_sp, trial_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
+                                    A.append(TPMatrix([DM[0], M[1]], test_sp, trial_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
+                                    A.append(TPMatrix(DM, test_sp, trial_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
 
     # At this point A contains all matrices of the form. The length of A is
     # the number of inner products. For each index into A there are ndim 1D
@@ -335,8 +391,8 @@ def inner(expr0, expr1, output_array=None, level=0):
     #           TPMatrix([(v[0], u[0])_x, (v[1], u[1]'')_y])]
     #
     # where v[0], v[1] are the test functions in x- and y-directions,
-    # respectively. For example, v[0] could be a ShenDirichletBasis and v[1]
-    # could be a FourierBasis. Same for u.
+    # respectively. For example, v[0] could be a ShenDirichlet and v[1]
+    # could be a Fourier space. Same for u.
     #
     # There are now two possibilities, either a linear or a bilinear form.
     # A linear form has trial.argument == 2, whereas a bilinear form has
@@ -348,17 +404,17 @@ def inner(expr0, expr1, output_array=None, level=0):
     # Considering again the tensor product space with ShenDirichlet and Fourier,
     # the list A will contain matrices as shown above. If Fourier is associated
     # with index 1, then (v[1], u[1])_y and (v[1], u[1]'')_y will be diagonal
-    # whereas (v[0], u[0]'')_x and (v[0], u[0])_x will in general not. These
-    # two matrices are usually termed the stiffness and mass matrices, and they
-    # have been implemented in chebyshev/matrices.py or legendre/matrices.py,
-    # where they are called ADDmat and BDDmat, respectively.
+    # whereas (v[0], u[0]'')_x and (v[0], u[0])_x will in general not.
 
     if level == 2 and trial.argument == 1: # No processing of matrices
         return A
 
     for tpmat in A:
         if isinstance(tpmat, TPMatrix):
-            tpmat.simplify_fourier_matrices()
+            try:
+                tpmat.simplify_diagonal_matrices()
+            except KeyError:
+                continue
 
     # Add equal matrices
     B = [A[0]]
@@ -380,7 +436,7 @@ def inner(expr0, expr1, output_array=None, level=0):
     # Linear form, return output_array
     wh = np.zeros_like(output_array)
     for b in A:
-        if uh.rank > 0:
+        if uh.function_space().is_composite_space:
             wh = b.matvec(uh.v[b.global_index[1]], wh)
         else:
             wh = b.matvec(uh, wh)
