@@ -34,7 +34,6 @@ def div(test):
     if isinstance(test, BasisFunction):
         test = Expr(test)
 
-    #test = copy.copy(test)
     ndim = test.dimensions
     coors = test.function_space().coors
 
@@ -52,7 +51,7 @@ def div(test):
                 for i in range(ndim):
                     dv.append(div(test[i]))
 
-                return _expr_from_vector_components(dv, test.base)
+                return _expr_from_vector_components(dv, test.basis())
 
             else: # vector
                 d = Dx(test[0], 0, 1)
@@ -63,36 +62,41 @@ def div(test):
 
     else:
 
-        if test.num_components() == ndim**2:
-
-            ct = coors.get_christoffel_second()
-            d = []
-            for i in range(ndim):
-                di = []
-                for j in range(ndim):
-                    Sij = test[i][j]
-                    di.append(Dx(Sij, j, 1))
-                    for k in range(ndim):
-                        Sik = test[i][k]
-                        Sjk = test[j][k]
-                        if not ct[i, k, j] == 0:
-                            di.append(Sjk*ct[i, k, j])
-                        if not ct[k, k, j] == 0:
-                            di.append(Sij*ct[k, k, j])
-
-                dj = di[0]
-                for j in range(1, len(di)):
-                    dj += di[j]
-                dj.simplify()
-                d.append(dj)
-            return _expr_from_vector_components(d, test.base)
+        if ndim == 1:      # 1D
+            sg = coors.get_sqrt_det_g()
+            d = Dx(test*sg, 0, 1)*(1/sg)
+            return d
 
         else:
-            sg = coors.get_sqrt_g()
-            d = Dx(test[0]*sg, 0, 1)*(1/sg)
-            for i in range(1, ndim):
-                d += Dx(test[i]*sg, i, 1)*(1/sg)
-        d.simplify()
+            if test.num_components() == ndim**2:
+
+                ct = coors.get_christoffel_second()
+                d = []
+                for i in range(ndim):
+                    di = []
+                    for j in range(ndim):
+                        Sij = test[i][j]
+                        di.append(Dx(Sij, j, 1))
+                        for k in range(ndim):
+                            Skj = test[k][j]
+                            if not ct[i, j, k] == 0:
+                                di.append(Skj*ct[i, j, k])
+                            if not ct[k, k, j] == 0:
+                                di.append(Sij*ct[k, k, j])
+
+                    dj = di[0]
+                    for j in range(1, len(di)):
+                        dj += di[j]
+                    dj.simplify()
+                    d.append(dj)
+                return _expr_from_vector_components(d, test.basis())
+
+            else:
+                sg = coors.get_sqrt_det_g()
+                d = Dx(test[0]*sg, 0, 1)*(1/sg)
+                for i in range(1, ndim):
+                    d += Dx(test[i]*sg, i, 1)*(1/sg)
+            d.simplify()
         return d
 
 def grad(test):
@@ -112,7 +116,6 @@ def grad(test):
     if isinstance(test, BasisFunction):
         test = Expr(test)
 
-    #test = copy.copy(test)
     ndim = test.dimensions
     coors = test.function_space().coors
 
@@ -128,11 +131,10 @@ def grad(test):
                d.append(Dx(test, i, 1))
 
     else:
-        #assert test.expr_rank() < 2, 'Cannot (yet) take gradient of higher order tensor in curvilinear coordinates'
 
         gt = coors.get_contravariant_metric_tensor()
 
-        if test.num_components() == ndim:
+        if test.num_components() > 1:
             ct = coors.get_christoffel_second()
             d = []
             for i in range(ndim):
@@ -145,7 +147,7 @@ def grad(test):
                             dj.append(Dx(vi, l, 1)*sc)
                         for k in range(ndim):
                             if not sc*ct[i, k, l] == 0:
-                                dj.append(test[k]*(sc*ct[i, j, k]))
+                                dj.append(test[k]*(sc*ct[i, k, l]))
 
                     di = dj[0]
                     for j in range(1, len(dj)):
@@ -165,7 +167,7 @@ def grad(test):
                     di += dj[j]
                 d.append(di)
 
-    dv = _expr_from_vector_components(d, test.base)
+    dv = _expr_from_vector_components(d, test.basis())
     dv.simplify()
     return dv
 
@@ -182,6 +184,11 @@ def Dx(test, x, k=1):
     """
     assert isinstance(test, (Expr, BasisFunction))
 
+    if k > 1:
+        for _ in range(k):
+            test = Dx(test, x, 1)
+        return test
+
     if isinstance(test, BasisFunction):
         test = Expr(test)
 
@@ -203,6 +210,8 @@ def Dx(test, x, k=1):
         for i in range(test.num_components()):
             for j in range(num_terms[i]):
                 sc0 = sp.simplify(sp.diff(sc[i][j], psi[x], k))
+                sc0 = sp.refine(sc0, coors._assumptions)
+
                 if not sc0 == 0:
                     v[i].append(copy.deepcopy(v[i][j]))
                     sc[i].append(sc0)
@@ -249,20 +258,47 @@ def curl(test):
     else:
         assert test.expr_rank() < 2, 'Cannot (yet) take curl of higher order tensor in curvilinear coordinates'
         hi = coors.hi
-        sg = coors.get_sqrt_g()
+        sg = coors.get_sqrt_det_g()
         if coors.is_orthogonal:
             if test.dimensions == 3:
                 w0 = (Dx(test[2]*hi[2]**2, 1, 1) - Dx(test[1]*hi[1]**2, 2, 1))*(1/sg)
                 w1 = (Dx(test[0]*hi[0]**2, 2, 1) - Dx(test[2]*hi[2]**2, 0, 1))*(1/sg)
                 w2 = (Dx(test[1]*hi[1]**2, 0, 1) - Dx(test[0]*hi[0]**2, 1, 1))*(1/sg)
-                test._terms = w0.terms()+w1.terms()+w2.terms()
-                test._scales = w0.scales()+w1.scales()+w2.scales()
-                test._indices = w0.indices()+w1.indices()+w2.indices()
-
+                test = _expr_from_vector_components([w0, w1, w2], test.basis())
             else:
                 assert test.dimensions == 2
                 test = (Dx(test[1]*hi[1]**2, 0, 1) - Dx(test[0]*hi[0]**2, 1, 1))*(1/sg)
         else:
-            raise NotImplementedError
+            g = coors.get_covariant_metric_tensor()
 
+            if test.dimensions == 3:
+                w0 = np.sum([(Dx(test[i]*g[2, i], 1, 1) - Dx(test[i]*g[1, i], 2, 1))*(1/sg) for i in range(3)])
+                w1 = np.sum([(Dx(test[i]*g[0, i], 2, 1) - Dx(test[i]*g[2, i], 0, 1))*(1/sg) for i in range(3)])
+                w2 = np.sum([(Dx(test[i]*g[1, i], 0, 1) - Dx(test[i]*g[0, i], 1, 1))*(1/sg) for i in range(3)])
+                # Don't think this double loop is needed due to symmetry of Christoffel?
+                #ct = coors.get_christoffel_second()
+                #for i in range(3):
+                #    for k in range(3):
+                #        w0 += (ct[i, 1, 2]*g[i, k]*test[k] - ct[i, 2, 1]*g[i, k]*test[k])*(1/sg)
+                #        w1 += (ct[i, 2, 0]*g[i, k]*test[k] - ct[i, 0, 2]*g[i, k]*test[k])*(1/sg)
+                #        w2 += (ct[i, 0, 1]*g[i, k]*test[k] - ct[i, 1, 0]*g[i, k]*test[k])*(1/sg)
+
+                # This is an alternative (more complicated way):
+                #gt = coors.get_contravariant_metric_tensor()
+                #ww0 = grad(g[0, 0]*test[0] + g[0, 1]*test[1] + g[0, 2]*test[2])
+                #ww1 = grad(g[1, 0]*test[0] + g[1, 1]*test[1] + g[1, 2]*test[2])
+                #ww2 = grad(g[2, 0]*test[0] + g[2, 1]*test[1] + g[2, 2]*test[2])
+                #d0 = sg*(ww0[1]*gt[0, 2] + ww1[1]*gt[1, 2] + ww2[1]*gt[2, 2] - ww0[2]*gt[0, 1] - ww1[2]*gt[1, 1] - ww2[2]*gt[2, 1])
+                #d1 = sg*(ww0[2]*gt[0, 0] + ww1[2]*gt[1, 0] + ww2[2]*gt[2, 0] - ww0[0]*gt[0, 2] - ww1[0]*gt[1, 2] - ww2[0]*gt[2, 2])
+                #d2 = sg*(ww0[0]*gt[0, 1] + ww1[0]*gt[1, 1] + ww2[0]*gt[2, 1] - ww0[1]*gt[0, 0] - ww1[1]*gt[1, 0] - ww2[1]*gt[2, 0])
+                #w0 = d0*gt[0, 0] + d1*gt[1, 0] + d2*gt[2, 0]
+                #w1 = d0*gt[0, 1] + d1*gt[1, 1] + d2*gt[2, 1]
+                #w2 = d0*gt[0, 2] + d1*gt[1, 2] + d2*gt[2, 2]
+
+                test = _expr_from_vector_components([w0, w1, w2], test.basis())
+            else:
+                assert test.dimensions == 2
+                test = np.sum([(Dx(test[i]*g[1, i], 0, 1) - Dx(test[i]*g[0, i], 1, 1))*(1/sg) for i in range(2)])
+
+    test.simplify()
     return test
