@@ -2,7 +2,7 @@ from copy import copy
 import types
 import numpy as np
 from shenfun.tensorproductspace import TensorProductSpace
-from shenfun.matrixbase import TPMatrix
+from shenfun.matrixbase import TPMatrix, BlockMatrix, SpectralMatrix
 from .arguments import Expr, TestFunction, TrialFunction, BasisFunction, \
     Function, Array
 from .inner import inner
@@ -27,7 +27,7 @@ def project(uh, T, output_array=None, fill=True, use_to_ortho=True, use_assign=T
         - :class:`.BasisFunction`
         - :class:`.Array`
         - A sympy function
-    T : :class:`.TensorProductSpace` or :class:`.MixedTensorProductSpace`
+    T : :class:`.TensorProductSpace` or :class:`.CompositeSpace`
     output_array : :class:`.Function`, optional
         Return array
     fill : bool, optional
@@ -53,12 +53,12 @@ def project(uh, T, output_array=None, fill=True, use_to_ortho=True, use_assign=T
 
     >>> import numpy as np
     >>> from mpi4py import MPI
-    >>> from shenfun import Basis, project, TensorProductSpace, Array, \
+    >>> from shenfun import FunctionSpace, project, TensorProductSpace, Array, \
     ...     Function, Dx
     >>> N = 16
     >>> comm = MPI.COMM_WORLD
-    >>> T0 = Basis(N, 'C')
-    >>> K0 = Basis(N, 'F', dtype='d')
+    >>> T0 = FunctionSpace(N, 'C')
+    >>> K0 = FunctionSpace(N, 'F', dtype='d')
     >>> T = TensorProductSpace(comm, (T0, K0))
     >>> uj = Array(T)
     >>> uj[:] = np.random.random(uj.shape)
@@ -82,6 +82,7 @@ def project(uh, T, output_array=None, fill=True, use_to_ortho=True, use_assign=T
 
     if isinstance(uh, Function):
         W = uh.function_space()
+
         if W == T:
             output_array[:] = uh
             return output_array
@@ -106,7 +107,7 @@ def project(uh, T, output_array=None, fill=True, use_to_ortho=True, use_assign=T
                 pass
 
     if isinstance(uh, np.ndarray) and not isinstance(uh, (Array, Function)):
-        assert np.all(uh.shape == T.shape(False))
+        #assert np.all(uh.shape == T.shape(False))
         uh = Array(T, buffer=uh)
 
     if isinstance(uh, Array):
@@ -150,10 +151,15 @@ def project(uh, T, output_array=None, fill=True, use_to_ortho=True, use_assign=T
             transAB.backward(output_arrayB2, output_array)
             return output_array
 
-    # Just zero or one non-periodic direction
-    if v.rank == 0:
+    if isinstance(B, (TPMatrix, SpectralMatrix)):
         output_array = B.solve(output_array, output_array)
-    else:
-        for oa, b in zip(output_array, B):
+
+    elif T.coors.is_orthogonal and (len(output_array) == len(B)):
+        for oa, b in zip(output_array.v, B):
             oa = b.solve(oa, oa)
+
+    else:
+        M = BlockMatrix(B)
+        output_array = M.solve(output_array, output_array)
+
     return output_array
