@@ -1,18 +1,17 @@
 """
-Module for defining bases in the Fourier family
+Module for defining function spaces in the Fourier family
 """
 import sympy as sp
 import numpy as np
 from mpi4py_fft import fftw
 from shenfun.spectralbase import SpectralBase, Transform, islicedict, slicedict
-from shenfun.utilities import inheritdocstrings
 from shenfun.optimization.cython import convolve
 
-__all__ = ['FourierBase', 'R2CBasis', 'C2CBasis']
+__all__ = ['FourierBase', 'R2C', 'C2C']
 
 #pylint: disable=method-hidden, no-member, line-too-long, arguments-differ
 
-@inheritdocstrings
+
 class FourierBase(SpectralBase):
     r"""Fourier base class
 
@@ -80,7 +79,7 @@ class FourierBase(SpectralBase):
         dealias_direct : bool, optional
             True for dealiasing using 2/3-rule. Must be used with
             padding_factor = 1.
-        coordinates: 2-tuple (coordinate, position vector), optional
+        coordinates: 2- or 3-tuple (coordinate, position vector (, sympy assumptions)), optional
             Map for curvilinear coordinatesystem.
             The new coordinate variable in the new coordinate system is the first item.
             Second item is a tuple for the Cartesian position vector as function of the
@@ -90,7 +89,7 @@ class FourierBase(SpectralBase):
                 rv = (sp.cos(theta), sp.sin(theta))
     """
 
-    def __init__(self, N, padding_factor=1., domain=(0, 2*np.pi), dtype=np.float,
+    def __init__(self, N, padding_factor=1, domain=(0, 2*np.pi), dtype=np.float,
                  dealias_direct=False, coordinates=None):
         self._k = None
         self._planned_axes = None  # Collapsing of axes means that this base can be used to plan transforms over several collapsed axes. Store the axes planned for here.
@@ -106,17 +105,20 @@ class FourierBase(SpectralBase):
 
     def points_and_weights(self, N=None, map_true_domain=False, weighted=True, **kw):
         if N is None:
-            N = self.N
+            N = self.shape(False)
         points = np.arange(N, dtype=np.float)*2*np.pi/N
         if map_true_domain is True:
             points = self.map_true_domain(points)
+        if weighted:
+            return points, np.array([1/N])
         return points, np.array([2*np.pi/N])
 
-    def sympy_basis(self, i=0, x=sp.symbols('x')):
-        return sp.exp(1j*i*x)
+    def sympy_basis(self, i=0, x=sp.symbols('x', real=True)):
+        k = self.wavenumbers(False, False, False)
+        return sp.exp(sp.I*k[i]*x)
 
-    def sympy_weight(self, x=sp.symbols('x')):
-        return 2*sp.pi/self.N
+    def weight(self, x=sp.symbols('x', real=True)):
+        return 1/(2*sp.pi)
 
     def evaluate_basis(self, x=None, i=0, output_array=None):
         if x is None:
@@ -150,11 +152,7 @@ class FourierBase(SpectralBase):
         return V
 
     # Reimplemented for efficiency (smaller array in *= when truncated)
-    def forward(self, input_array=None, output_array=None, fast_transform=True, padding_factor=1):
-        if padding_factor > 1:
-            assert self._padded_basis is not None
-            output_array = self._padded_basis.forward(input_array, output_array, fast_transform=fast_transform)
-            return output_array
+    def forward(self, input_array=None, output_array=None, fast_transform=True):
 
         if fast_transform is False:
             return SpectralBase.forward(self, input_array, output_array, False)
@@ -168,48 +166,30 @@ class FourierBase(SpectralBase):
         M = self.get_normalization()
         self.forward._output_array *= M
 
+        self.apply_inverse_mass(self.forward.output_array)
         if output_array is not None:
             output_array[...] = self.forward.output_array
             return output_array
         return self.forward.output_array
 
     def apply_inverse_mass(self, array):
-        """Apply inverse mass
-
-        Note
-        ----
-        Mass matrix is identity, so do nothing
-
-        Parameters
-        ----------
-            array : array (input/output)
-                    Expansion coefficients.
-        """
+        if not self.coors.is_cartesian: # mass matrix may not be diagonal
+            return SpectralBase.apply_inverse_mass(self, array)
         return array
 
-    def evaluate_expansion_all(self, input_array, output_array, fast_transform=True):
+    def _evaluate_scalar_product(self, fast_transform=True):
         if fast_transform is False:
-            SpectralBase.evaluate_expansion_all(self, input_array, output_array, False)
-        else:
-            self.backward.xfftn(normalise_idft=False)
-        assert input_array is self.backward.xfftn.input_array
-
-    def evaluate_scalar_product(self, input_array, output_array, fast_transform=True):
-        if fast_transform is False:
-            self.vandermonde_scalar_product(input_array, output_array)
+            SpectralBase._evaluate_scalar_product(self)
             return
         output = self.scalar_product.xfftn()
-        M = self.get_normalization()
-        output *= M
-        assert input_array is self.scalar_product.xfftn.input_array
-
-    def vandermonde_scalar_product(self, input_array, output_array):
-        SpectralBase.vandermonde_scalar_product(self, input_array, output_array)
-        output_array *= 0.5/np.pi
+        output *= self.get_normalization()
 
     def reference_domain(self):
         return (0., 2*np.pi)
 
+    def sympy_reference_domain(self):
+        return (0, 2*sp.pi)
+
     @property
     def is_orthogonal(self):
         return True
@@ -217,6 +197,13 @@ class FourierBase(SpectralBase):
     def get_orthogonal(self):
         return self
 
+    def get_unplanned(self):
+        return self.__class__(self.N,
+                              domain=self.domain,
+                              padding_factor=self.padding_factor,
+                              dealias_direct=self.dealias_direct,
+                              coordinates=self.coors.coordinates)
+
     def get_dealiased(self, padding_factor=1.5, dealias_direct=False):
         return self.__class__(self.N,
                               domain=self.domain,
@@ -249,6 +236,9 @@ class FourierBase(SpectralBase):
         return u_hat
 
     def plan(self, shape, axis, dtype, options):
+        if shape in (0, (0,)):
+            return
+
         if isinstance(axis, int):
             axis = [axis]
         s = list(np.take(shape, axis))
@@ -313,12 +303,10 @@ class FourierBase(SpectralBase):
             trunc_array = self._get_truncarray(shape, V.dtype)
             self.forward = Transform(self.forward, xfftn_fwd, U, V, trunc_array)
             self.backward = Transform(self.backward, xfftn_bck, trunc_array, V, U)
-            self.backward_uniform = self.backward
 
         else:
             self.forward = Transform(self.forward, xfftn_fwd, U, V, V)
             self.backward = Transform(self.backward, xfftn_bck, V, V, U)
-            self.backward_uniform = self.backward
 
         # scalar_product is not padded, just the forward/backward
         self.scalar_product = Transform(self.scalar_product, xfftn_fwd, U, V, V)
@@ -326,8 +314,8 @@ class FourierBase(SpectralBase):
         self.sl = slicedict(axis=self.axis, dimensions=self.dimensions)
 
 
-class R2CBasis(FourierBase):
-    """Fourier basis class for real to complex transforms
+class R2C(FourierBase):
+    """Fourier function space for real to complex transforms
 
     Parameters
     ----------
@@ -342,7 +330,7 @@ class R2CBasis(FourierBase):
         dealias_direct : bool, optional
             True for dealiasing using 2/3-rule. Must be used with
             padding_factor = 1.
-        coordinates: 2-tuple (coordinate, position vector), optional
+        coordinates: 2- or 3-tuple (coordinate, position vector (, sympy assumptions)), optional
             Map for curvilinear coordinatesystem.
             The new coordinate variable in the new coordinate system is the first item.
             Second item is a tuple for the Cartesian position vector as function of the
@@ -358,8 +346,6 @@ class R2CBasis(FourierBase):
                              domain=domain, dealias_direct=dealias_direct,
                              coordinates=coordinates)
         self.N = N
-        #self._xfftn_fwd = pyfftw.builders.rfftn
-        #self._xfftn_bck = pyfftw.builders.irfftn
         self._xfftn_fwd = fftw.rfftn
         self._xfftn_bck = fftw.irfftn
         self._sn = []
@@ -408,69 +394,31 @@ class R2CBasis(FourierBase):
             return self.N//2+1
         return int(np.floor(self.padding_factor*self.N))
 
-    def vandermonde_evaluate_expansion_all(self, input_array, output_array, x=None):
-        assert abs(self.padding_factor-1) < 1e-8
-        assert self.N == output_array.shape[self.axis]
-        points = self.points_and_weights()[0]
-        P = self.vandermonde(points)
-        if output_array.ndim == 1:
-            output_array[:] = np.dot(P, input_array).real
-            if self.N % 2 == 0:
-                output_array += np.conj(np.dot(P[:, 1:-1], input_array[1:-1])).real
+    def evaluate_expansion_all(self, input_array, output_array, x=None, fast_transform=True):
+        if fast_transform is False:
+            assert abs(self.padding_factor-1) < 1e-8
+            P = self.evaluate_basis_all(x=x)
+            if output_array.ndim == 1:
+                output_array[:] = np.dot(P, input_array).real
+                if self.N % 2 == 0:
+                    output_array += np.conj(np.dot(P[:, 1:-1], input_array[1:-1])).real
+                else:
+                    output_array += np.conj(np.dot(P[:, 1:], input_array[1:])).real
+
             else:
-                output_array += np.conj(np.dot(P[:, 1:], input_array[1:])).real
+                fc = np.moveaxis(input_array, self.axis, -2)
+                array = np.dot(P, fc).real
+                s = [slice(None)]*fc.ndim
+                if self.N % 2 == 0:
+                    s[-2] = slice(1, -1)
+                    array += np.conj(np.dot(P[:, 1:-1], fc[tuple(s)])).real
+                else:
+                    s[-2] = slice(1, None)
+                    array += np.conj(np.dot(P[:, 1:], fc[tuple(s)])).real
 
+                output_array[:] = np.moveaxis(array, 0, self.axis)
         else:
-            fc = np.moveaxis(input_array, self.axis, -2)
-            array = np.dot(P, fc).real
-            s = [slice(None)]*fc.ndim
-            if self.N % 2 == 0:
-                s[-2] = slice(1, -1)
-                array += np.conj(np.dot(P[:, 1:-1], fc[tuple(s)])).real
-            else:
-                s[-2] = slice(1, None)
-                array += np.conj(np.dot(P[:, 1:], fc[tuple(s)])).real
-
-            output_array[:] = np.moveaxis(array, 0, self.axis)
-
-    def vandermonde_evaluate_expansion(self, points, input_array, output_array):
-        """Evaluate expansion at certain points, possibly different from
-        the quadrature points
-
-        This method assumes the array is locally available in full, i.e., the
-        multidimensional arrays are aligned along the axis of this basis.
-
-        Parameters
-        ----------
-            P : 2D array
-                Vandermode matrix containing local points only
-            input_array : array
-                          Expansion coefficients
-            output_array : array
-                           Function values on points
-            last_conj_index : int
-                              The last index to sum over for conj part
-                              (R2CBasis only)
-            offset : int
-                     Global offset (MPI)
-
-        Note
-        ----
-        This method is complicated by the fact that the data may not be aligned
-        along the axis of this basis.
-
-        """
-        assert abs(self.padding_factor-1) < 1e-8
-        points = self.map_reference_domain(points)
-        P = self.evaluate_basis_all(points)
-        assert output_array.ndim == 1 # Multidimensional should use vandermonde_evaluate_local_expansion
-        output_array[:] = np.dot(P, input_array).real
-        if self.N % 2 == 0:
-            output_array += np.conj(np.dot(P[:, 1:-1], input_array[1:-1])).real
-        else:
-            output_array += np.conj(np.dot(P[:, 1:], input_array[1:])).real
-
-        return output_array
+            self.backward.xfftn(normalise_idft=False)
 
     def _truncation_forward(self, padded_array, trunc_array):
         if not id(trunc_array) == id(padded_array):
@@ -541,8 +489,8 @@ class R2CBasis(FourierBase):
         return uv
 
 
-class C2CBasis(FourierBase):
-    """Fourier basis class for complex to complex transforms
+class C2C(FourierBase):
+    """Fourier function space for complex to complex transforms
 
     Parameters
     ----------
@@ -557,7 +505,7 @@ class C2CBasis(FourierBase):
         dealias_direct : bool, optional
             True for dealiasing using 2/3-rule. Must be used with
             padding_factor = 1.
-        coordinates: 2-tuple (coordinate, position vector), optional
+        coordinates: 2- or 3-tuple (coordinate, position vector (, sympy assumptions)), optional
             Map for curvilinear coordinatesystem.
             The new coordinate variable in the new coordinate system is the first item.
             Second item is a tuple for the Cartesian position vector as function of the
@@ -567,19 +515,23 @@ class C2CBasis(FourierBase):
                 rv = (sp.cos(theta), sp.sin(theta))
     """
 
-    def __init__(self, N, padding_factor=1., domain=(0., 2.*np.pi),
+    def __init__(self, N, padding_factor=1, domain=(0., 2.*np.pi),
                  dealias_direct=False, coordinates=None):
         FourierBase.__init__(self, N, padding_factor=padding_factor, dtype=np.complex,
                              domain=domain, dealias_direct=dealias_direct,
                              coordinates=coordinates)
         self.N = N
-        #self._xfftn_fwd = pyfftw.builders.fftn
-        #self._xfftn_bck = pyfftw.builders.ifftn
         self._xfftn_fwd = fftw.fftn
         self._xfftn_bck = fftw.ifftn
         self.plan((int(padding_factor*N),), (0,), np.complex, {})
         self._slp = []
 
+    def evaluate_expansion_all(self, input_array, output_array, x=None, fast_transform=True):
+        if fast_transform is False:
+            SpectralBase.evaluate_expansion_all(self, input_array, output_array, x, False)
+        else:
+            self.backward.xfftn(normalise_idft=False)
+
     def wavenumbers(self, bcast=True, scaled=False, eliminate_highest_freq=False):
         k = np.fft.fftfreq(self.N, 1./self.N).astype(int)
         if self.N % 2 == 0 and eliminate_highest_freq:
@@ -608,6 +560,13 @@ class C2CBasis(FourierBase):
             return self.N
         return int(np.floor(self.padding_factor*self.N))
 
+    def count_trailing_zeros(self, u, reltol=1e-12, abstol=1e-15):
+        assert u.function_space() == self
+        assert u.ndim == 1
+        a = abs(u[self.slice()])
+        ua = (a < reltol*a.max()) | (a < abstol)
+        return np.argmin(ua[self.N//2:]) + np.argmin(ua[:self.N//2][::-1])
+
     def _truncation_forward(self, padded_array, trunc_array):
         if not id(trunc_array) == id(padded_array):
             trunc_array.fill(0)
