@@ -5,14 +5,12 @@ from numpy.polynomial import hermite
 from scipy.special import eval_hermite, factorial
 from mpi4py_fft import fftw
 from shenfun.spectralbase import SpectralBase, Transform, islicedict, slicedict
-from shenfun.utilities import inheritdocstrings
 
 #pylint: disable=method-hidden,no-else-return,not-callable,abstract-method,no-member,cyclic-import
 
 
-@inheritdocstrings
-class Basis(SpectralBase):
-    """Base class for Hermite functions
+class Orthogonal(SpectralBase):
+    r"""Base class for Hermite functions
 
     Parameters
     ----------
@@ -35,7 +33,7 @@ class Basis(SpectralBase):
         dtype : data-type, optional
             Type of input data in real physical space. Will be overloaded when
             basis is part of a :class:`.TensorProductSpace`.
-        coordinates: 2-tuple (coordinate, position vector), optional
+        coordinates: 2- or 3-tuple (coordinate, position vector (, sympy assumptions)), optional
             Map for curvilinear coordinatesystem.
             The new coordinate variable in the new coordinate system is the first item.
             Second item is a tuple for the Cartesian position vector as function of the
@@ -57,7 +55,6 @@ class Basis(SpectralBase):
     polynomials of order k, respectively.
 
     """
-
     def __init__(self, N, quad="HG", bc=(0., 0.), dtype=np.float,
                  padding_factor=1, dealias_direct=False, coordinates=None):
         SpectralBase.__init__(self, N, quad=quad, domain=(-np.inf, np.inf), dtype=dtype,
@@ -98,9 +95,17 @@ class Basis(SpectralBase):
                               dealias_direct=dealias_direct,
                               coordinates=self.coors.coordinates)
 
+    def get_unplanned(self):
+        return self.__class__(self.N,
+                              quad=self.quad,
+                              dtype=self.dtype,
+                              padding_factor=self.padding_factor,
+                              dealias_direct=self.dealias_direct,
+                              coordinates=self.coors.coordinates)
+
     def points_and_weights(self, N=None, map_true_domain=False, weighted=True, **kw):
         if N is None:
-            N = self.N
+            N = self.shape(False)
         if self.quad == "HG":
             points, weights = hermite.hermgauss(N)
             if weighted:
@@ -115,7 +120,7 @@ class Basis(SpectralBase):
         return 1./(np.pi**(0.25)*np.sqrt(2.**i)*np.sqrt(factorial(i)))
 
     def vandermonde(self, x):
-        V = hermite.hermvander(x, self.N-1)
+        V = hermite.hermvander(x, self.shape(False)-1)
         return V
 
     def sympy_basis(self, i=0, x=sp.symbols('x')):
@@ -173,6 +178,9 @@ class Basis(SpectralBase):
         return V
 
     def plan(self, shape, axis, dtype, options):
+        if shape in (0, (0,)):
+            return
+
         if isinstance(axis, tuple):
             assert len(axis) == 1
             axis = axis[0]
@@ -191,11 +199,9 @@ class Basis(SpectralBase):
             trunc_array = self._get_truncarray(shape, V.dtype)
             self.forward = Transform(self.forward, None, U, V, trunc_array)
             self.backward = Transform(self.backward, None, trunc_array, V, U)
-            self.backward_uniform = Transform(self.backward_uniform, None, trunc_array, V, U)
         else:
             self.forward = Transform(self.forward, None, U, V, V)
             self.backward = Transform(self.backward, None, V, V, U)
-            self.backward_uniform = Transform(self.backward_uniform, None, V, V, U)
         self.scalar_product = Transform(self.scalar_product, None, U, V, V)
         self.si = islicedict(axis=self.axis, dimensions=self.dimensions)
         self.sl = slicedict(axis=self.axis, dimensions=self.dimensions)
