@@ -1,5 +1,5 @@
 """
-Module for bases of generalized Jacobi type
+Module for function spaces of generalized Jacobi type
 
 Note the environment variable
 
@@ -35,8 +35,8 @@ import sympy as sp
 from scipy.special import eval_jacobi, roots_jacobi #, gamma
 from mpi4py_fft import fftw
 from shenfun.spectralbase import SpectralBase, Transform, islicedict, slicedict
-from shenfun.utilities import inheritdocstrings
 from shenfun.forms.arguments import Function
+from shenfun.chebyshev.bases import BCBiharmonic, BCDirichlet
 
 try:
     import quadpy
@@ -50,14 +50,16 @@ except:
 mode = os.environ.get('SHENFUN_JACOBI_MODE', 'numpy')
 mode = mode if has_quadpy else 'numpy'
 
+xp = sp.Symbol('x', real=True)
+
 #pylint: disable=method-hidden,no-else-return,not-callable,abstract-method,no-member,cyclic-import
 
-__all__ = ['JacobiBase', 'Basis', 'ShenDirichletBasis', 'ShenBiharmonicBasis',
-           'ShenOrder6Basis', 'mode', 'has_quadpy', 'mp']
+__all__ = ['JacobiBase', 'Orthogonal', 'ShenDirichlet', 'ShenBiharmonic',
+           'ShenOrder6', 'mode', 'has_quadpy', 'mp']
+
 
-@inheritdocstrings
 class JacobiBase(SpectralBase):
-    """Base class for all Jacobi bases
+    """Base class for all Jacobi spaces
 
     Parameters
     ----------
@@ -80,7 +82,7 @@ class JacobiBase(SpectralBase):
         dtype : data-type, optional
             Type of input data in real physical space. Will be overloaded when
             basis is part of a :class:`.TensorProductSpace`.
-        coordinates: 2-tuple (coordinate, position vector), optional
+        coordinates: 2- or 3-tuple (coordinate, position vector (, sympy assumptions)), optional
             Map for curvilinear coordinatesystem.
             The new coordinate variable in the new coordinate system is the first item.
             Second item is a tuple for the Cartesian position vector as function of the
@@ -101,13 +103,14 @@ class JacobiBase(SpectralBase):
         self.forward = functools.partial(self.forward, fast_transform=False)
         self.backward = functools.partial(self.backward, fast_transform=False)
         self.scalar_product = functools.partial(self.scalar_product, fast_transform=False)
+        self.plan(int(N*padding_factor), 0, dtype, {})
 
     @staticmethod
     def family():
         return 'jacobi'
 
     def reference_domain(self):
-        return (-1., 1.)
+        return (-1, 1)
 
     def get_refined(self, N):
         return self.__class__(N,
@@ -131,9 +134,20 @@ class JacobiBase(SpectralBase):
                               alpha=self.alpha,
                               beta=self.beta)
 
+    def get_orthogonal(self):
+        return Orthogonal(self.N,
+                          quad=self.quad,
+                          domain=self.domain,
+                          dtype=self.dtype,
+                          padding_factor=self.padding_factor,
+                          dealias_direct=self.dealias_direct,
+                          coordinates=self.coors.coordinates,
+                          alpha=0,
+                          beta=0)
+
     def points_and_weights(self, N=None, map_true_domain=False, weighted=True, **kw):
         if N is None:
-            N = self.N
+            N = self.shape(False)
         assert self.quad == "JG"
         points, weights = roots_jacobi(N, self.alpha, self.beta)
         if map_true_domain is True:
@@ -144,10 +158,10 @@ class JacobiBase(SpectralBase):
         if mode == 'numpy' or not has_quadpy:
             return self.points_and_weights(N=N, map_true_domain=map_true_domain, weighted=weighted, **kw)
         if N is None:
-            N = self.N
-        pw = quadpy.line_segment.gauss_jacobi(N, self.alpha, self.beta, 'mpmath')
-        points = pw.points
-        weights = pw.weights
+            N = self.shape(False)
+        pw = quadpy.c1.gauss_jacobi(N, self.alpha, self.beta, 'mpmath')
+        points = pw.points_symbolic
+        weights = pw.weights_symbolic
         if map_true_domain is True:
             points = self.map_true_domain(points)
         return points, weights
@@ -158,9 +172,8 @@ class JacobiBase(SpectralBase):
             for n in range(N):
                 V[:, n] = eval_jacobi(n, alpha, beta, x)
         else:
-            X = sp.symbols('x', real=True)
             for n in range(N):
-                V[:, n] = sp.lambdify(X, sp.jacobi(n, alpha, beta, X), 'mpmath')(x)
+                V[:, n] = sp.lambdify(xp, sp.jacobi(n, alpha, beta, xp), 'mpmath')(x)
         return V
 
     def derivative_jacobi(self, x, alpha, beta, k=1):
@@ -175,9 +188,12 @@ class JacobiBase(SpectralBase):
         return V
 
     def vandermonde(self, x):
-        return self.jacobi(x, self.alpha, self.beta, self.N)
+        return self.jacobi(x, self.alpha, self.beta, self.shape(False))
 
     def plan(self, shape, axis, dtype, options):
+        if shape in (0, (0,)):
+            return
+
         if isinstance(axis, tuple):
             assert len(axis) == 1
             axis = axis[0]
@@ -192,19 +208,20 @@ class JacobiBase(SpectralBase):
         U.fill(0)
         V.fill(0)
         self.axis = axis
-        self.forward = Transform(self.forward, None, U, V, V)
-        self.backward = Transform(self.backward, None, V, V, U)
-        self.backward_uniform = Transform(self.backward_uniform, None, V, V, U)
+        if self.padding_factor > 1.+1e-8:
+            trunc_array = self._get_truncarray(shape, V.dtype)
+            self.forward = Transform(self.forward, None, U, V, trunc_array)
+            self.backward = Transform(self.backward, None, trunc_array, V, U)
+        else:
+            self.forward = Transform(self.forward, None, U, V, V)
+            self.backward = Transform(self.backward, None, V, V, U)
         self.scalar_product = Transform(self.scalar_product, None, U, V, V)
         self.si = islicedict(axis=self.axis, dimensions=self.dimensions)
         self.sl = slicedict(axis=self.axis, dimensions=self.dimensions)
 
-    def get_orthogonal(self):
-        return Basis(self.N, alpha=self.alpha, beta=self.beta, domain=self.domain)
 
-@inheritdocstrings
-class Basis(JacobiBase):
-    """Basis for regular (orthogonal) Jacobi functions
+class Orthogonal(JacobiBase):
+    """Function space for regular (orthogonal) Jacobi functions
 
     Parameters
     ----------
@@ -222,7 +239,7 @@ class Basis(JacobiBase):
         dtype : data-type, optional
             Type of input data in real physical space. Will be overloaded when
             basis is part of a :class:`.TensorProductSpace`.
-        coordinates: 2-tuple (coordinate, position vector), optional
+        coordinates: 2- or 3-tuple (coordinate, position vector (, sympy assumptions)), optional
             Map for curvilinear coordinatesystem.
             The new coordinate variable in the new coordinate system is the first item.
             Second item is a tuple for the Cartesian position vector as function of the
@@ -237,16 +254,15 @@ class Basis(JacobiBase):
         JacobiBase.__init__(self, N, quad=quad, alpha=alpha, beta=beta, domain=domain, dtype=dtype,
                             padding_factor=padding_factor, dealias_direct=dealias_direct,
                             coordinates=coordinates)
-        self.plan(int(N*padding_factor), 0, dtype, {})
 
     @property
     def is_orthogonal(self):
         return True
 
-    def get_orthogonal(self):
-        return self
+    #def get_orthogonal(self):
+    #    return self
 
-    def sympy_basis(self, i=0, x=sp.symbols('x', real=True)):
+    def sympy_basis(self, i=0, x=xp):
         return sp.jacobi(i, self.alpha, self.beta, x)
 
     def evaluate_basis(self, x, i=0, output_array=None):
@@ -256,29 +272,10 @@ class Basis(JacobiBase):
         if mode == 'numpy':
             output_array = eval_jacobi(i, self.alpha, self.beta, x, out=output_array)
         else:
-            X = sp.symbols('x', real=True)
-            f = self.sympy_basis(i, X)
-            output_array[:] = sp.lambdify(X, f, 'mpmath')(x)
+            f = self.sympy_basis(i, xp)
+            output_array[:] = sp.lambdify(xp, f, 'mpmath')(x)
         return output_array
 
-    def evaluate_basis_derivative_all(self, x=None, k=0, argument=0):
-        if x is None:
-            x = self.mpmath_points_and_weights(mode=mode)[0]
-        #if x.dtype == 'O':
-        #    x = np.array(x, dtype=self.dtype)
-        if mode == 'numpy':
-            return self.derivative_jacobi(x, self.alpha, self.beta, k)
-        else:
-            V = np.zeros((x.shape[0], self.N))
-            for i in range(self.N):
-                V[:, i] = self.evaluate_basis_derivative(x, i, k, output_array=V[:, i])
-        return V
-
-    def evaluate_basis_all(self, x=None, argument=0):
-        if x is None:
-            x = self.mpmath_points_and_weights()[0]
-        return self.vandermonde(x)
-
     def evaluate_basis_derivative(self, x=None, i=0, k=0, output_array=None):
         if x is None:
             x = self.points_and_weights(mode=mode)[0]
@@ -290,14 +287,32 @@ class Basis(JacobiBase):
             dj = np.prod(np.array([i+self.alpha+self.beta+1+j for j in range(k)]))
             output_array[:] = dj/2**k*eval_jacobi(i-k, self.alpha+k, self.beta+k, x)
         else:
-            X = sp.symbols('x', real=True)
-            f = sp.jacobi(i, self.alpha, self.beta, X)
-            output_array[:] = sp.lambdify(X, f.diff(X, k), 'mpmath')(x)
+            f = sp.jacobi(i, self.alpha, self.beta, xp)
+            output_array[:] = sp.lambdify(xp, f.diff(xp, k), 'mpmath')(x)
         return output_array
 
+    def evaluate_basis_derivative_all(self, x=None, k=0, argument=0):
+        if x is None:
+            x = self.mpmath_points_and_weights(mode=mode)[0]
+        #if x.dtype == 'O':
+        #    x = np.array(x, dtype=self.dtype)
+        if mode == 'numpy':
+            return self.derivative_jacobi(x, self.alpha, self.beta, k)
+        else:
+            N = self.shape(False)
+            V = np.zeros((x.shape[0], N))
+            for i in range(N):
+                V[:, i] = self.evaluate_basis_derivative(x, i, k, output_array=V[:, i])
+        return V
 
-class ShenDirichletBasis(JacobiBase):
-    """Jacobi basis for Dirichlet boundary conditions
+    def evaluate_basis_all(self, x=None, argument=0):
+        if x is None:
+            x = self.mpmath_points_and_weights()[0]
+        return self.vandermonde(x)
+
+
+class ShenDirichlet(JacobiBase):
+    """Jacobi function space for Dirichlet boundary conditions
 
     Parameters
     ----------
@@ -319,7 +334,7 @@ class ShenDirichletBasis(JacobiBase):
         dtype : data-type, optional
             Type of input data in real physical space. Will be overloaded when
             basis is part of a :class:`.TensorProductSpace`.
-        coordinates: 2-tuple (coordinate, position vector), optional
+        coordinates: 2- or 3-tuple (coordinate, position vector (, sympy assumptions)), optional
             Map for curvilinear coordinatesystem.
             The new coordinate variable in the new coordinate system is the first item.
             Second item is a tuple for the Cartesian position vector as function of the
@@ -336,8 +351,8 @@ class ShenDirichletBasis(JacobiBase):
                             coordinates=coordinates)
         assert bc in ((0, 0), 'Dirichlet')
         from shenfun.tensorproductspace import BoundaryValues
+        self._bc_basis = None
         self.bc = BoundaryValues(self, bc=bc)
-        self.plan(int(N*padding_factor), 0, dtype, {})
 
     @staticmethod
     def boundary_condition():
@@ -372,12 +387,13 @@ class ShenDirichletBasis(JacobiBase):
     def evaluate_basis_derivative_all(self, x=None, k=0, argument=0):
         if x is None:
             x = self.mpmath_points_and_weights()[0]
-        V = np.zeros((x.shape[0], self.N))
-        for i in range(self.N-2):
+        N = self.shape(False)
+        V = np.zeros((x.shape[0], N))
+        for i in range(N-2):
             V[:, i] = self.evaluate_basis_derivative(x, i, k, output_array=V[:, i])
         return V
 
-    def sympy_basis(self, i=0, x=sp.symbols('x', real=True)):
+    def sympy_basis(self, i=0, x=xp):
         return (1-x**2)*sp.jacobi(i, 1, 1, x)
         #return (1-x)**(-self.alpha)*(1+x)**(-self.beta)*sp.jacobi(i, -self.alpha, -self.beta, x)
 
@@ -386,9 +402,8 @@ class ShenDirichletBasis(JacobiBase):
             x = self.mpmath_points_and_weights()[0]
         if output_array is None:
             output_array = np.zeros(x.shape, dtype=self.dtype)
-        X = sp.symbols('x', real=True)
-        f = self.sympy_basis(i, X)
-        output_array[:] = sp.lambdify(X, f.diff(X, k), mode)(x)
+        f = self.sympy_basis(i, xp)
+        output_array[:] = sp.lambdify(xp, f.diff(xp, k), mode)(x)
         return output_array
 
     def evaluate_basis(self, x, i=0, output_array=None):
@@ -398,9 +413,8 @@ class ShenDirichletBasis(JacobiBase):
         if mode == 'numpy':
             output_array = (1-x**2)*eval_jacobi(i, -self.alpha, -self.beta, x, out=output_array)
         else:
-            X = sp.symbols('x', real=True)
-            f = self.sympy_basis(i, X)
-            output_array[:] = sp.lambdify(X, f, 'mpmath')(x)
+            f = self.sympy_basis(i, xp)
+            output_array[:] = sp.lambdify(xp, f, 'mpmath')(x)
         return output_array
 
     def evaluate_basis_all(self, x=None, argument=0):
@@ -412,14 +426,15 @@ class ShenDirichletBasis(JacobiBase):
         else:
             if x is None:
                 x = self.mpmath_points_and_weights()[0]
-            V = np.zeros((x.shape[0], self.N))
-            for i in range(self.N-2):
+            N = self.shape(False)
+            V = np.zeros((x.shape[0], N))
+            for i in range(N-2):
                 V[:, i] = self.evaluate_basis(x, i, output_array=V[:, i])
         return V
 
     def points_and_weights(self, N=None, map_true_domain=False, weighted=True, **kw):
         if N is None:
-            N = self.N
+            N = self.shape(False)
         assert self.quad == "JG"
         points, weights = roots_jacobi(N, self.alpha+1, self.beta+1)
         if map_true_domain is True:
@@ -430,11 +445,11 @@ class ShenDirichletBasis(JacobiBase):
         if mode == 'numpy' or not has_quadpy:
             return self.points_and_weights(N=N, map_true_domain=map_true_domain, weighted=weighted, **kw)
         if N is None:
-            N = self.N
+            N = self.shape(False)
         assert self.quad == "JG"
-        pw = quadpy.line_segment.gauss_jacobi(N, self.alpha+1, self.beta+1, mode)
-        points = pw.points
-        weights = pw.weights
+        pw = quadpy.c1.gauss_jacobi(N, self.alpha+1, self.beta+1, mode)
+        points = pw.points_symbolic
+        weights = pw.weights_symbolic
         if map_true_domain is True:
             points = self.map_true_domain(points)
         return points, weights
@@ -442,7 +457,7 @@ class ShenDirichletBasis(JacobiBase):
     def to_ortho(self, input_array, output_array=None):
         assert self.alpha == -1 and self.beta == -1
         if output_array is None:
-            output_array = Function(self.get_orthogonal())
+            output_array = np.zeros_like(input_array)
         else:
             output_array.fill(0)
         k = self.wavenumbers().astype(np.float)
@@ -453,15 +468,20 @@ class ShenDirichletBasis(JacobiBase):
         output_array[s1] -= z
         return output_array
 
-    def get_orthogonal(self):
-        return Basis(self.N, alpha=self.alpha+1, beta=self.beta+1, dtype=self.dtype, domain=self.domain, coordinates=self.coors.coordinates)
+    def _evaluate_scalar_product(self, fast_transform=True):
+        SpectralBase._evaluate_scalar_product(self)
+        self.scalar_product.output_array[self.sl[slice(-2, None)]] = 0
 
-    def vandermonde_scalar_product(self, input_array, output_array):
-        SpectralBase.vandermonde_scalar_product(self, input_array, output_array)
-        output_array[self.sl[slice(-2, None)]] = 0
+    def get_bc_basis(self):
+        if self._bc_basis:
+            return self._bc_basis
+        self._bc_basis = BCDirichlet(self.N, quad=self.quad, domain=self.domain,
+                                     coordinates=self.coors.coordinates)
+        return self._bc_basis
 
-class ShenBiharmonicBasis(JacobiBase):
-    """Basis for Biharmonic boundary conditions
+
+class ShenBiharmonic(JacobiBase):
+    """Function space for Biharmonic boundary conditions
 
     Parameters
     ----------
@@ -481,7 +501,7 @@ class ShenBiharmonicBasis(JacobiBase):
         dtype : data-type, optional
             Type of input data in real physical space. Will be overloaded when
             basis is part of a :class:`.TensorProductSpace`.
-        coordinates: 2-tuple (coordinate, position vector), optional
+        coordinates: 2- or 3-tuple (coordinate, position vector (, sympy assumptions)), optional
             Map for curvilinear coordinatesystem.
             The new coordinate variable in the new coordinate system is the first item.
             Second item is a tuple for the Cartesian position vector as function of the
@@ -497,12 +517,14 @@ class ShenBiharmonicBasis(JacobiBase):
 
     """
     def __init__(self, N, quad='JG', bc=(0, 0, 0, 0), domain=(-1., 1.), dtype=np.float,
-                 padding_factor=1, dealias_direct=False, coordinates=None):
+                 padding_factor=1, dealias_direct=False, coordinates=None, **kw):
         JacobiBase.__init__(self, N, quad=quad, alpha=-2, beta=-2, domain=domain, dtype=dtype,
                             padding_factor=padding_factor, dealias_direct=dealias_direct,
                             coordinates=coordinates)
         assert bc in ((0, 0, 0, 0), 'Biharmonic')
-        self.plan(int(N*padding_factor), 0, dtype, {})
+        from shenfun.tensorproductspace import BoundaryValues
+        self._bc_basis = None
+        self.bc = BoundaryValues(self, bc=bc)
 
     @staticmethod
     def boundary_condition():
@@ -511,14 +533,15 @@ class ShenBiharmonicBasis(JacobiBase):
     def slice(self):
         return slice(0, self.N-4)
 
-    def sympy_basis(self, i=0, x=sp.symbols('x', real=True)):
+    def sympy_basis(self, i=0, x=xp):
         return (1-x**2)**2*sp.jacobi(i, 2, 2, x)
 
     def evaluate_basis_derivative_all(self, x=None, k=0, argument=0):
         if x is None:
             x = self.mpmath_points_and_weights()[0]
-        V = np.zeros((x.shape[0], self.N))
-        for i in range(self.N-4):
+        N = self.shape(False)
+        V = np.zeros((x.shape[0], N))
+        for i in range(N-4):
             V[:, i] = self.evaluate_basis_derivative(x, i, k, output_array=V[:, i])
         return V
 
@@ -527,9 +550,8 @@ class ShenBiharmonicBasis(JacobiBase):
             x = self.mpmath_points_and_weights()[0]
         if output_array is None:
             output_array = np.zeros(x.shape, dtype=self.dtype)
-        X = sp.symbols('x', real=True)
-        f = self.sympy_basis(i, X)
-        output_array[:] = sp.lambdify(X, f.diff(X, k), mode)(x)
+        f = self.sympy_basis(i, xp)
+        output_array[:] = sp.lambdify(xp, f.diff(xp, k), mode)(x)
         return output_array
 
     def evaluate_basis(self, x, i=0, output_array=None):
@@ -539,32 +561,33 @@ class ShenBiharmonicBasis(JacobiBase):
         if mode == 'numpy':
             output_array[:] = (1-x**2)**2*eval_jacobi(i, 2, 2, x, out=output_array)
         else:
-            X = sp.symbols('x', real=True)
-            f = self.sympy_basis(i, X)
-            output_array[:] = sp.lambdify(X, f, 'mpmath')(x)
+            f = self.sympy_basis(i, xp)
+            output_array[:] = sp.lambdify(xp, f, 'mpmath')(x)
         return output_array
 
     def evaluate_basis_all(self, x=None, argument=0):
         if mode == 'numpy':
             if x is None:
                 x = self.mesh(False, False)
-            V = np.zeros((x.shape[0], self.N))
-            V[:, :-4] = self.jacobi(x, 2, 2, self.N-4)*((1-x**2)**2)[:, np.newaxis]
+            N = self.shape(False)
+            V = np.zeros((x.shape[0], N))
+            V[:, :-4] = self.jacobi(x, 2, 2, N-4)*((1-x**2)**2)[:, np.newaxis]
         else:
             if x is None:
                 x = self.mpmath_points_and_weights()[0]
-            V = np.zeros((x.shape[0], self.N))
-            for i in range(self.N-4):
+            N = self.shape(False)
+            V = np.zeros((x.shape[0], N))
+            for i in range(N-4):
                 V[:, i] = self.evaluate_basis(x, i, output_array=V[:, i])
         return V
 
-    def vandermonde_scalar_product(self, input_array, output_array):
-        SpectralBase.vandermonde_scalar_product(self, input_array, output_array)
-        output_array[self.sl[slice(-4, None)]] = 0
+    def _evaluate_scalar_product(self, fast_transform=True):
+        SpectralBase._evaluate_scalar_product(self)
+        self.scalar_product.output_array[self.sl[slice(-4, None)]] = 0
 
     def points_and_weights(self, N=None, map_true_domain=False, weighted=True, **kw):
         if N is None:
-            N = self.N
+            N = self.shape(False)
         assert self.quad == "JG"
         points, weights = roots_jacobi(N, 0, 0)
         if map_true_domain is True:
@@ -575,18 +598,18 @@ class ShenBiharmonicBasis(JacobiBase):
         if mode == 'numpy' or not has_quadpy:
             return self.points_and_weights(N=N, map_true_domain=map_true_domain, weighted=weighted, **kw)
         if N is None:
-            N = self.N
+            N = self.shape(False)
         assert self.quad == "JG"
-        pw = quadpy.line_segment.gauss_jacobi(N, 0, 0, 'mpmath')
-        points = pw.points
-        weights = pw.weights
+        pw = quadpy.c1.gauss_jacobi(N, 0, 0, 'mpmath')
+        points = pw.points_symbolic
+        weights = pw.weights_symbolic
         if map_true_domain is True:
             points = self.map_true_domain(points)
         return points, weights
 
     def to_ortho(self, input_array, output_array=None):
         if output_array is None:
-            output_array = Function(self.get_orthogonal())
+            output_array = np.zeros_like(input_array)
         else:
             output_array.fill(0)
         k = self.wavenumbers().astype(np.float)
@@ -600,12 +623,16 @@ class ShenBiharmonicBasis(JacobiBase):
         output_array[self.sl[slice(4, None)]] += z*_factor2
         return output_array
 
-    def get_orthogonal(self):
-        return Basis(self.N, alpha=0, beta=0, dtype=self.dtype, domain=self.domain, coordinates=self.coors.coordinates)
+    def get_bc_basis(self):
+        if self._bc_basis:
+            return self._bc_basis
+        self._bc_basis = BCBiharmonic(self.N, quad=self.quad, domain=self.domain,
+                                      coordinates=self.coors.coordinates)
+        return self._bc_basis
 
 
-class ShenOrder6Basis(JacobiBase):
-    """Basis for 6th order equation
+class ShenOrder6(JacobiBase):
+    """Function space for 6th order equation
 
     Parameters
     ----------
@@ -625,7 +652,7 @@ class ShenOrder6Basis(JacobiBase):
         dtype : data-type, optional
             Type of input data in real physical space. Will be overloaded when
             basis is part of a :class:`.TensorProductSpace`.
-        coordinates: 2-tuple (coordinate, position vector), optional
+        coordinates: 2- or 3-tuple (coordinate, position vector (, sympy assumptions)), optional
             Map for curvilinear coordinatesystem.
             The new coordinate variable in the new coordinate system is the first item.
             Second item is a tuple for the Cartesian position vector as function of the
@@ -645,7 +672,8 @@ class ShenOrder6Basis(JacobiBase):
         JacobiBase.__init__(self, N, quad=quad, alpha=-3, beta=-3, domain=domain, dtype=dtype,
                             padding_factor=padding_factor, dealias_direct=dealias_direct,
                             coordinates=coordinates)
-        self.plan(int(N*padding_factor), 0, dtype, {})
+        from shenfun.tensorproductspace import BoundaryValues
+        self.bc = BoundaryValues(self, bc=(0,)*6)
 
     @staticmethod
     def boundary_condition():
@@ -654,7 +682,7 @@ class ShenOrder6Basis(JacobiBase):
     def slice(self):
         return slice(0, self.N-6)
 
-    def sympy_basis(self, i=0, x=sp.symbols('x', real=True)):
+    def sympy_basis(self, i=0, x=xp):
         return (1-x**2)**3*sp.jacobi(i, 3, 3, x)
 
     def evaluate_basis_derivative(self, x=None, i=0, k=0, output_array=None):
@@ -662,16 +690,16 @@ class ShenOrder6Basis(JacobiBase):
             x = self.mpmath_points_and_weights()[0]
         if output_array is None:
             output_array = np.zeros(x.shape)
-        X = sp.symbols('x', real=True)
-        f = self.sympy_basis(i, X)
-        output_array[:] = sp.lambdify(X, f.diff(X, k), mode)(x)
+        f = self.sympy_basis(i, xp)
+        output_array[:] = sp.lambdify(xp, f.diff(xp, k), mode)(x)
         return output_array
 
     def evaluate_basis_derivative_all(self, x=None, k=0, argument=0):
         if x is None:
             x = self.mpmath_points_and_weights()[0]
-        V = np.zeros((x.shape[0], self.N))
-        for i in range(self.N-6):
+        N = self.shape(False)
+        V = np.zeros((x.shape[0], N))
+        for i in range(N-6):
             V[:, i] = self.evaluate_basis_derivative(x, i, k, output_array=V[:, i])
         return V
 
@@ -682,28 +710,29 @@ class ShenOrder6Basis(JacobiBase):
         if mode == 'numpy':
             output_array[:] = (1-x**2)**3*eval_jacobi(i, 3, 3, x, out=output_array)
         else:
-            X = sp.symbols('x', real=True)
-            f = self.sympy_basis(i, X)
-            output_array[:] = sp.lambdify(X, f, 'mpmath')(x)
+            f = self.sympy_basis(i, xp)
+            output_array[:] = sp.lambdify(xp, f, 'mpmath')(x)
         return output_array
 
     def evaluate_basis_all(self, x=None, argument=0):
         if mode == 'numpy':
             if x is None:
                 x = self.mesh(False, False)
-            V = np.zeros((x.shape[0], self.N))
-            V[:, :-6] = self.jacobi(x, 3, 3, self.N-6)*((1-x**2)**3)[:, np.newaxis]
+            N = self.shape(False)
+            V = np.zeros((x.shape[0], N))
+            V[:, :-6] = self.jacobi(x, 3, 3, N-6)*((1-x**2)**3)[:, np.newaxis]
         else:
             if x is None:
                 x = self.mpmath_points_and_weights()[0]
-            V = np.zeros((x.shape[0], self.N))
-            for i in range(self.N-6):
+            N = self.shape(False)
+            V = np.zeros((x.shape[0], N))
+            for i in range(N-6):
                 V[:, i] = self.evaluate_basis(x, i, output_array=V[:, i])
         return V
 
     def points_and_weights(self, N=None, map_true_domain=False, weighted=True, **kw):
         if N is None:
-            N = self.N
+            N = self.shape(False)
         assert self.quad == "JG"
         points, weights = roots_jacobi(N, 0, 0)
         if map_true_domain is True:
@@ -714,21 +743,21 @@ class ShenOrder6Basis(JacobiBase):
         if mode == 'numpy' or not has_quadpy:
             return self.points_and_weights(N=N, map_true_domain=map_true_domain, weighted=weighted, **kw)
         if N is None:
-            N = self.N
+            N = self.shape(False)
         assert self.quad == "JG"
-        pw = quadpy.line_segment.gauss_jacobi(N, 0, 0, 'mpmath')
-        points = pw.points
-        weights = pw.weights
+        pw = quadpy.c1.gauss_jacobi(N, 0, 0, 'mpmath')
+        points = pw.points_symbolic
+        weights = pw.weights_symbolic
         if map_true_domain is True:
             points = self.map_true_domain(points)
         return points, weights
 
     def get_orthogonal(self):
-        return Basis(self.N, alpha=0, beta=0, dtype=self.dtype, domain=self.domain, coordinates=self.coors.coordinates)
+        return Orthogonal(self.N, alpha=0, beta=0, dtype=self.dtype, domain=self.domain, coordinates=self.coors.coordinates)
 
-    def vandermonde_scalar_product(self, input_array, output_array):
-        SpectralBase.vandermonde_scalar_product(self, input_array, output_array)
-        output_array[self.sl[slice(-6, None)]] = 0
+    def _evaluate_scalar_product(self, fast_transform=True):
+        SpectralBase._evaluate_scalar_product(self)
+        self.scalar_product.output_array[self.sl[slice(-6, None)]] = 0
 
     #def to_ortho(self, input_array, output_array=None):
     #    if output_array is None:
