@@ -2,12 +2,12 @@ r"""
 This module contains linear algebra solvers for SparseMatrixes
 """
 import numpy as np
-import scipy.sparse as sp
-from scipy.sparse.linalg import spsolve
+import scipy.sparse as scp
+from scipy.sparse.linalg import spsolve, splu
 from shenfun.optimization import optimizer
 from shenfun.matrixbase import SparseMatrix
 
-class TDMA(object):
+class TDMA:
     """Tridiagonal matrix solver
 
     Parameters
@@ -56,7 +56,7 @@ class TDMA(object):
         for i in range(n - 3, -1, -1):
             x[i] = (x[i] - a[i]*x[i+2])/d[i]
 
-    def __call__(self, b, u=None, axis=0):
+    def __call__(self, b, u=None, axis=0, **kw):
         """Solve matrix problem self u = b
 
         Parameters
@@ -89,7 +89,7 @@ class TDMA(object):
         u /= self.mat.scale
         return u
 
-class PDMA(object):
+class PDMA:
     """Pentadiagonal matrix solver
 
     Parameters
@@ -176,7 +176,7 @@ class PDMA(object):
             bc[k] -= (e[k]*bc[k+2] + f[k]*bc[k+4])
         b[:] = bc.astype(float)
 
-    def __call__(self, b, u=None, axis=0):
+    def __call__(self, b, u=None, axis=0, **kw):
         """Solve matrix problem self u = b
 
         Parameters
@@ -208,7 +208,7 @@ class PDMA(object):
         u /= self.mat.scale
         return u
 
-class Solve(object):
+class Solve:
     """Solver class for matrix created by Dirichlet bases
 
     Possibly with inhomogeneous boundary values
@@ -226,7 +226,7 @@ class Solve(object):
         self.A = A
         self.test = test
 
-    def __call__(self, b, u=None, axis=0):
+    def __call__(self, b, u=None, axis=0, use_lu=False):
         """Solve matrix problem Au = b
 
         Parameters
@@ -257,30 +257,44 @@ class Solve(object):
 
         s = self.s
         assert self.A.shape[0] == b[s].shape[0]
-        A = self.A.diags('csr')
+        A = self.A.diags('csc')
         if b.ndim == 1:
-            u[s] = spsolve(A, b[s])
+            if use_lu:
+                if b.dtype.char in 'FDG':
+                    u.real[s] = self._lu.solve(b[s].real)
+                    u.imag[s] = self._lu.solve(b[s].imag)
+                else:
+                    u[s] = self._lu.solve(b[s])
+            else:
+                u[s] = spsolve(A, b[s])
+            #u[s] = spsolve(A, b[s])
+
         else:
             N = b[s].shape[0]
             P = np.prod(b[s].shape[1:])
             br = b[s].reshape((N, P))
-
-            if b.dtype is np.dtype('complex'):
-                u.real[s] = spsolve(A, br.real).reshape(u[s].shape)
-                u.imag[s] = spsolve(A, br.imag).reshape(u[s].shape)
+            if use_lu:
+                if b.dtype.char in 'FDG':
+                    u.real[s] = self._lu.solve(br.real).reshape(u[s].shape)
+                    u.imag[s] = self._lu.solve(br.imag).reshape(u[s].shape)
+                else:
+                    u[s] = self._lu.solve(br).reshape(u[s].shape)
             else:
                 u[s] = spsolve(A, br).reshape(u[s].shape)
-        #if self.test.has_nonhomogeneous_bcs:
-        #    self.test.bc.set_boundary_dofs(u, True)
+
+            #if b.dtype is np.dtype('complex'):
+            #    u.real[s] = spsolve(A, br.real).reshape(u[s].shape)
+            #    u.imag[s] = spsolve(A, br.imag).reshape(u[s].shape)
+            #else:
+            #    u[s] = spsolve(A, br).reshape(u[s].shape)
 
         if axis > 0:
             u = np.moveaxis(u, 0, axis)
             if u is not b:
                 b = np.moveaxis(b, 0, axis)
-
         return u
 
-class NeumannSolve(object):
+class NeumannSolve:
     """Solver class for matrix created by Neumann bases
 
     Assuming Neumann test- and trialfunction, where index k=0 is used only
@@ -299,7 +313,7 @@ class NeumannSolve(object):
         self.s = test.slice()
         self.A = A
 
-    def __call__(self, b, u=None, axis=0):
+    def __call__(self, b, u=None, axis=0, use_lu=False):
         """Solve matrix problem A u = b
 
         Parameters
@@ -334,15 +348,26 @@ class NeumannSolve(object):
         A[0, 0] = 1
 
         if b.ndim == 1:
-            u[s] = spsolve(A, b[s])
+            if use_lu:
+                if b.dtype.char in 'FDG':
+                    u.real[s] = self._lu.solve(b[s].real)
+                    u.imag[s] = self._lu.solve(b[s].imag)
+                else:
+                    u[s] = self._lu.solve(b[s])
+            else:
+                u[s] = spsolve(A, b[s])
+
         else:
             N = b[s].shape[0]
             P = np.prod(b[s].shape[1:])
             br = b[s].reshape((N, P))
 
-            if b.dtype is np.dtype('complex'):
-                u.real[s] = spsolve(A, br.real).reshape(u[s].shape)
-                u.imag[s] = spsolve(A, br.imag).reshape(u[s].shape)
+            if use_lu:
+                if b.dtype.char in 'FDG':
+                    u.real[s] = self._lu.solve(br.real).reshape(u[s].shape)
+                    u.imag[s] = self._lu.solve(br.imag).reshape(u[s].shape)
+                else:
+                    u[s] = self._lu.solve(br).reshape(u[s].shape)
             else:
                 u[s] = spsolve(A, br).reshape(u[s].shape)
 
@@ -350,12 +375,11 @@ class NeumannSolve(object):
             u = np.moveaxis(u, 0, axis)
             if u is not b:
                 b = np.moveaxis(b, 0, axis)
-        #u /= self.A.scale
         return u
 
-class SolverGeneric2NP(object):
-    """Generic solver for tensorproductspaces consisting of (currently) two
-    non-periodic bases.
+class SolverGeneric2ND:
+    """Generic solver for problems consisting of tensorproduct matrices
+    containing two non-diagonal submatrices.
 
     Parameters
     ----------
@@ -364,25 +388,14 @@ class SolverGeneric2NP(object):
 
     Note
     ----
-    In addition to two non-periodic directions, the solver can also handle one
-    periodic direction.
+    In addition to two non-diagonal matrices, the solver can also handle one
+    additional diagonal matrix (one Fourier matrix).
     """
 
-    def __init__(self, mats):
-        self.mats = mats
-        m = mats[0]
-        #assert len(m.naxes) == 2
-        self.T = T = m.space
-        ndim = T.dimensions
-        if ndim == 2:
-            m = mats[0]
-            M0 = sp.kron(m.mats[0].diags(), m.mats[1].diags())
-            M0 *= np.atleast_1d(m.scale).item()
-            for m in mats[1:]:
-                M1 = sp.kron(m.mats[0].diags(), m.mats[1].diags())
-                M1 *= np.atleast_1d(m.scale).item()
-                M0 = M0 + M1
-            self.M = M0
+    def __init__(self, tpmats):
+        self.tpmats = tpmats
+        self.T = T = tpmats[0].space
+        self.M = None
 
     def matvec(self, u, c):
         c.fill(0)
@@ -391,38 +404,66 @@ class SolverGeneric2NP(object):
             c[s0] = self.M.dot(u[s0].flatten()).reshape(self.T.dims())
         return c
 
-    def __call__(self, b, u=None):
+    def get_diagonal_axis(self):
+        naxes = self.T.get_nondiagonal_axes()
+        diagonal_axis = np.setxor1d([0, 1, 2], naxes)
+        assert len(diagonal_axis) == 1
+        return diagonal_axis[0]
+
+    def diags(self, i, format='csr'):
+        """Return matrix for given index `i` in diagonal direction"""
+        if self.T.dimensions == 2:
+            # In 2D there's just 1 matrix, store and reuse
+            if self.M is not None:
+                return self.M
+            m = self.tpmats[0]
+            M0 = scp.kron(m.mats[0].diags(format), m.mats[1].diags(format))
+            M0 *= np.atleast_1d(m.scale).item()
+            for m in self.tpmats[1:]:
+                M1 = scp.kron(m.mats[0].diags(format), m.mats[1].diags(format))
+                M1 *= np.atleast_1d(m.scale).item()
+                M0 = M0 + M1
+            self.M = M0
+            return self.M
+
+        else:
+            # 1 matrix per Fourier coefficient
+            naxes = self.T.get_nondiagonal_axes()
+            m = self.tpmats[0]
+            diagonal_axis = self.get_diagonal_axis()
+            sc = [0, 0, 0]
+            sc[diagonal_axis] = i if m.scale.shape[diagonal_axis] > 1 else 0
+            M0 = scp.kron(m.mats[naxes[0]].diags(format), m.mats[naxes[1]].diags(format))
+            M0 *= m.scale[tuple(sc)]
+            for m in self.tpmats[1:]:
+                M1 = scp.kron(m.mats[naxes[0]].diags(format), m.mats[naxes[1]].diags(format))
+                sc[diagonal_axis] = i if m.scale.shape[diagonal_axis] > 1 else 0
+                M1 *= m.scale[tuple(sc)]
+                M0 = M0 + M1
+            return M0
+
+    def __call__(self, b, u=None, format='csr'):
         if u is None:
             u = b
         else:
             assert u.shape == b.shape
         if u.ndim == 2:
-            s0 = tuple(base.slice() for base in self.T)
-            u[s0] = sp.linalg.spsolve(self.M, b[s0].flatten()).reshape(self.T.dims())
+            s0 = self.T.slice()
+            M = self.diags(0, format=format)
+            u[s0] = scp.linalg.spsolve(M, b[s0].flatten()).reshape(self.T.dims())
+
         elif u.ndim == 3:
-            naxes = self.T.get_nonperiodic_axes()
-            periodic_axis = np.setxor1d([0, 1, 2], naxes)
-            assert len(periodic_axis) == 1
-            periodic_axis = periodic_axis[0]
-            M = self.T.shape(True)[periodic_axis]
-            sc = [0, 0, 0]
-            for i in range(M):
-                m = self.mats[0]
-                M0 = sp.kron(m.mats[naxes[0]].diags(), m.mats[naxes[1]].diags())
-                sc[periodic_axis] = i if m.scale.shape[periodic_axis] > 1 else 0
-                M0 *= m.scale[tuple(sc)]
-                for m in self.mats[1:]:
-                    M1 = sp.kron(m.mats[naxes[0]].diags(), m.mats[naxes[1]].diags())
-                    sc[periodic_axis] = i if m.scale.shape[periodic_axis] > 1 else 0
-                    M1 *= m.scale[tuple(sc)]
-                    M0 = M0 + M1
-                s0 = [base.slice() for base in self.T]
-                s0[periodic_axis] = i
+            naxes = self.T.get_nondiagonal_axes()
+            diagonal_axis = self.get_diagonal_axis()
+            s0 = list(self.T.slice())
+            for i in range(self.T.shape(True)[diagonal_axis]):
+                M0 = self.diags(i, format=format)
+                s0[diagonal_axis] = i
                 shape = np.take(self.T.dims(), naxes)
-                u[tuple(s0)] = sp.linalg.spsolve(M0, b[tuple(s0)].flatten()).reshape(shape)
+                u[tuple(s0)] = scp.linalg.spsolve(M0, b[tuple(s0)].flatten()).reshape(shape)
         return u
 
-class Solver2D(object):
+class Solver2D:
     """Generic solver for tensorproductspaces in 2D
 
     Parameters
@@ -432,17 +473,17 @@ class Solver2D(object):
 
     """
 
-    def __init__(self, mats):
-        self.mats = mats
-        m = mats[0]
+    def __init__(self, tpmats):
+        self.tpmats = tpmats
+        m = tpmats[0]
         self.T = T = m.space
         ndim = T.dimensions
         assert ndim == 2
         assert np.atleast_1d(m.scale).size == 1, "Use level = 2 with :func:`.inner`"
-        M0 = sp.kron(m.mats[0].diags(), m.mats[1].diags())
+        M0 = scp.kron(m.mats[0].diags(), m.mats[1].diags())
         M0 *= np.atleast_1d(m.scale).item()
-        for m in mats[1:]:
-            M1 = sp.kron(m.mats[0].diags(), m.mats[1].diags())
+        for m in tpmats[1:]:
+            M1 = scp.kron(m.mats[0].diags(), m.mats[1].diags())
             assert np.atleast_1d(m.scale).size == 1, "Use level = 2 with :func:`.inner`"
             M1 *= np.atleast_1d(m.scale).item()
             M0 = M0 + M1
@@ -460,11 +501,11 @@ class Solver2D(object):
         else:
             assert u.shape == b.shape
         s0 = tuple(base.slice() for base in self.T)
-        u[s0] = sp.linalg.spsolve(self.M, b[s0].flatten()).reshape(self.T.dims())
+        u[s0] = scp.linalg.spsolve(self.M, b[s0].flatten()).reshape(self.T.dims())
         return u
 
 
-class TDMA_O(object):
+class TDMA_O:
     """Tridiagonal matrix solver
 
     Parameters
@@ -546,9 +587,9 @@ class TDMA_O(object):
         return u
 
 
-class SolverGeneric1NP(object):
-    """Generic solver for tensorproductspaces consisting of only one
-    non-periodic space.
+class SolverGeneric1ND:
+    """Generic solver for tensorproduct matrices consisting of
+    non-diagonal matrices along only one axis.
 
     Parameters
     ----------
@@ -557,9 +598,9 @@ class SolverGeneric1NP(object):
 
     Note
     ----
-    In addition to the one non-periodic direction, the solver can also handle
-    up to two periodic directions. Also, this Python version of the solver is
-    not very efficient. Consider using Cython.
+    In addition to the one non-diagonal direction, the solver can also handle
+    up to two diagonal (Fourier) directions. Also, this Python version of the
+    solver is not very efficient. Consider implementing in Cython.
 
     """
 
@@ -568,10 +609,14 @@ class SolverGeneric1NP(object):
         m = mats[0]
         if m.naxes == []:
             for tpmat in mats:
-                tpmat.simplify_fourier_matrices()
+                tpmat.simplify_diagonal_matrices()
 
         self.mats = mats
 
+        # For time-dependent solver, store all generated matrices and reuse
+        # This takes a lot of memory, so for now it's only implemented for 2D
+        self.MM = None
+
     def __call__(self, b, u=None):
         if u is None:
             u = b
@@ -581,34 +626,54 @@ class SolverGeneric1NP(object):
 
         if u.ndim == 2:
             if m.naxes[0] == 0:
-                # non-periodic in axis=0
-                for i in range(b.shape[1]):
-                    MM = None
-                    for mat in self.mats:
-                        sc = mat.scale[0, i] if mat.scale.shape[1] > 1 else mat.scale[0, 0]
-                        if MM:
-                            MM += sc*mat.mats[0]
-                        else:
-                            MM = sc*mat.mats[0]
-                    sl = mat.space.bases[0].slice()
-                    u[sl, i] = MM.solve(b[sl, i], u[sl, i])
+                # non-diagonal in axis=0
+                if self.MM is None:
+                    self.MM = []
+                    for i in range(b.shape[1]):
+                        MM = None
+                        for mat in self.mats:
+                            sc = mat.scale[0, i] if mat.scale.shape[1] > 1 else mat.scale[0, 0]
+                            if MM:
+                                MM += sc*mat.mats[0]
+                            else:
+                                MM = sc*mat.mats[0]
+                        sl = m.space.bases[0].slice()
+                        MM._lu = splu(MM.diags('csc'))
+                        u[sl, i] = MM.solve(b[sl, i], u[sl, i], use_lu=True)
+                        self.MM.append(MM)
+
+                else:
+                    for i in range(b.shape[1]):
+                        sl = m.space.bases[0].slice()
+                        u[sl, i] = self.MM[i].solve(b[sl, i], u[sl, i], use_lu=True)
 
             else:
-                # non-periodic in axis=1
-                for i in range(b.shape[0]):
-                    MM = None
-                    for mat in self.mats:
-                        sc = mat.scale[i, 0] if mat.scale.shape[0] > 1 else mat.scale[0, 0]
-                        if MM:
-                            MM += sc*mat.mats[1]
-                        else:
-                            MM = sc*mat.mats[1]
-                    sl = mat.space.bases[1].slice()
-                    u[i, sl] = MM.solve(b[i, sl], u[i, sl])
+                if self.MM is None:
+                    # non-diagonal in axis=1
+                    self.MM = []
+                    for i in range(b.shape[0]):
+                        MM = None
+                        for mat in self.mats:
+                            sc = mat.scale[i, 0] if mat.scale.shape[0] > 1 else mat.scale[0, 0]
+                            if MM:
+                                MM += sc*mat.mats[1]
+                            else:
+                                MM = sc*mat.mats[1]
+                        sl = m.space.bases[1].slice()
+                        u[i, sl] = MM.solve(b[i, sl], u[i, sl])
+                        MM._lu = splu(MM.diags('csc'))
+                        MM.solve(b[i, sl], u[i, sl], use_lu=True)
+                        self.MM.append(MM)
+
+                else:
+                    for i in range(b.shape[0]):
+                        sl = m.space.bases[1].slice()
+                        u[i, sl] = self.MM[i].solve(b[i, sl], u[i, sl], use_lu=True)
+
 
         elif u.ndim == 3:
             if m.naxes[0] == 0:
-                # non-periodic in axis=0
+                # non-diagonal in axis=0
                 for i in range(b.shape[1]):
                     for j in range(b.shape[2]):
                         MM = None
@@ -622,7 +687,7 @@ class SolverGeneric1NP(object):
                         u[sl, i, j] = MM.solve(b[sl, i, j], u[sl, i, j])
 
             elif m.naxes[0] == 1:
-                # non-periodic in axis=1
+                # non-diagonal in axis=1
                 for i in range(b.shape[0]):
                     for j in range(b.shape[2]):
                         MM = None
@@ -636,7 +701,7 @@ class SolverGeneric1NP(object):
                         u[i, sl, j] = MM.solve(b[i, sl, j], u[i, sl, j])
 
             elif m.naxes[0] == 2:
-                # non-periodic in axis=2
+                # non-diagonal in axis=2
                 for i in range(b.shape[0]):
                     for j in range(b.shape[1]):
                         MM = None
