@@ -5,15 +5,13 @@ from numpy.polynomial import laguerre as lag
 from scipy.special import eval_laguerre
 from mpi4py_fft import fftw
 from shenfun.spectralbase import SpectralBase, work, Transform, islicedict, slicedict
-from shenfun.utilities import inheritdocstrings
 from shenfun.forms.arguments import Function
 
 #pylint: disable=method-hidden,no-else-return,not-callable,abstract-method,no-member,cyclic-import
 
 
-@inheritdocstrings
 class LaguerreBase(SpectralBase):
-    r"""Base class for all Laguerre bases
+    r"""Base class for all Laguerre spaces
 
     Parameters
     ----------
@@ -31,7 +29,7 @@ class LaguerreBase(SpectralBase):
         dtype : data-type, optional
             Type of input data in real physical space. Will be overloaded when
             basis is part of a :class:`.TensorProductSpace`.
-        coordinates: 2-tuple (coordinate, position vector), optional
+        coordinates: 2- or 3-tuple (coordinate, position vector (, sympy assumptions)), optional
             Map for curvilinear coordinatesystem.
             The new coordinate variable in the new coordinate system is the first item.
             Second item is a tuple for the Cartesian position vector as function of the
@@ -58,6 +56,7 @@ class LaguerreBase(SpectralBase):
         self.forward = functools.partial(self.forward, fast_transform=False)
         self.backward = functools.partial(self.backward, fast_transform=False)
         self.scalar_product = functools.partial(self.scalar_product, fast_transform=False)
+        self.plan(int(N*padding_factor), 0, dtype, {})
 
     @staticmethod
     def family():
@@ -71,7 +70,7 @@ class LaguerreBase(SpectralBase):
 
     def points_and_weights(self, N=None, map_true_domain=False, weighted=True, **kw):
         if N is None:
-            N = self.N
+            N = self.shape(False)
         if self.quad == "LG":
             points, weights = lag.laggauss(N)
             if weighted:
@@ -82,7 +81,7 @@ class LaguerreBase(SpectralBase):
         return points, weights
 
     def vandermonde(self, x):
-        V = lag.lagvander(x, self.N-1)
+        V = lag.lagvander(x, int(self.N*self.padding_factor)-1)
         return V
 
     def evaluate_basis(self, x, i=0, output_array=None):
@@ -119,20 +118,23 @@ class LaguerreBase(SpectralBase):
         else:
             raise NotImplementedError
 
-        return self._composite_basis(V)
+        return self._composite(V)
 
     def evaluate_basis_all(self, x=None, argument=0):
         if x is None:
             x = self.mesh(False, False)
         V = self.vandermonde(x)
         V *= np.exp(-x/2)[:, np.newaxis]
-        return self._composite_basis(V, argument)
+        return self._composite(V, argument)
 
-    def _composite_basis(self, V, argument=0):
+    def _composite(self, V, argument=0):
         """Return composite basis, where ``V`` is primary Vandermonde matrix."""
         return V
 
     def plan(self, shape, axis, dtype, options):
+        if shape in (0, (0,)):
+            return
+
         if isinstance(axis, tuple):
             assert len(axis) == 1
             axis = axis[0]
@@ -147,8 +149,13 @@ class LaguerreBase(SpectralBase):
         U.fill(0)
         V.fill(0)
         self.axis = axis
-        self.forward = Transform(self.forward, None, U, V, V)
-        self.backward = Transform(self.backward, None, V, V, U)
+        if self.padding_factor > 1.+1e-8:
+            trunc_array = self._get_truncarray(shape, V.dtype)
+            self.forward = Transform(self.forward, None, U, V, trunc_array)
+            self.backward = Transform(self.backward, None, trunc_array, V, U)
+        else:
+            self.forward = Transform(self.forward, None, U, V, V)
+            self.backward = Transform(self.backward, None, V, V, U)
         self.scalar_product = Transform(self.scalar_product, None, U, V, V)
         self.si = islicedict(axis=self.axis, dimensions=self.dimensions)
         self.sl = slicedict(axis=self.axis, dimensions=self.dimensions)
@@ -169,9 +176,17 @@ class LaguerreBase(SpectralBase):
                               dealias_direct=dealias_direct,
                               coordinates=self.coors.coordinates)
 
-@inheritdocstrings
-class Basis(LaguerreBase):
-    r"""Basis for regular Laguerre functions
+    def get_unplanned(self):
+        return self.__class__(self.N,
+                              quad=self.quad,
+                              dtype=self.dtype,
+                              padding_factor=self.padding_factor,
+                              dealias_direct=self.dealias_direct,
+                              coordinates=self.coors.coordinates)
+
+
+class Orthogonal(LaguerreBase):
+    r"""Function space for regular Laguerre functions
 
     Parameters
     ----------
@@ -188,7 +203,7 @@ class Basis(LaguerreBase):
         dtype : data-type, optional
             Type of input data in real physical space. Will be overloaded when
             basis is part of a :class:`.TensorProductSpace`.
-        coordinates: 2-tuple (coordinate, position vector), optional
+        coordinates: 2- or 3-tuple (coordinate, position vector (, sympy assumptions)), optional
             Map for curvilinear coordinatesystem.
             The new coordinate variable in the new coordinate system is the first item.
             Second item is a tuple for the Cartesian position vector as function of the
@@ -213,7 +228,6 @@ class Basis(LaguerreBase):
     def __init__(self, N, quad="LG", dtype=np.float, padding_factor=1, dealias_direct=False, coordinates=None):
         LaguerreBase.__init__(self, N, quad=quad, dtype=dtype, padding_factor=padding_factor,
                               dealias_direct=dealias_direct, coordinates=coordinates)
-        self.plan(int(N*padding_factor), 0, dtype, {})
 
     def eval(self, x, u, output_array=None):
         x = np.atleast_1d(x)
@@ -232,9 +246,9 @@ class Basis(LaguerreBase):
     def get_orthogonal(self):
         return self
 
-@inheritdocstrings
-class ShenDirichletBasis(LaguerreBase):
-    """Shen Laguerre basis for Dirichlet boundary conditions
+
+class ShenDirichlet(LaguerreBase):
+    """Laguerre function space for Dirichlet boundary conditions
 
     Parameters
     ----------
@@ -251,7 +265,7 @@ class ShenDirichletBasis(LaguerreBase):
         dtype : data-type, optional
             Type of input data in real physical space. Will be overloaded when
             basis is part of a :class:`.TensorProductSpace`.
-        coordinates: 2-tuple (coordinate, position vector), optional
+        coordinates: 2- or 3-tuple (coordinate, position vector (, sympy assumptions)), optional
             Map for curvilinear coordinatesystem.
             The new coordinate variable in the new coordinate system is the first item.
             Second item is a tuple for the Cartesian position vector as function of the
@@ -265,14 +279,12 @@ class ShenDirichletBasis(LaguerreBase):
                  dealias_direct=False, coordinates=None):
         LaguerreBase.__init__(self, N, dtype=dtype, quad=quad, padding_factor=padding_factor,
                               dealias_direct=dealias_direct, coordinates=coordinates)
-        self.LT = Basis(N, quad)
-        self.plan(int(N*padding_factor), 0, dtype, {})
 
     @staticmethod
     def boundary_condition():
         return 'Dirichlet'
 
-    def _composite_basis(self, V, argument=0):
+    def _composite(self, V, argument=0):
         assert self.N == V.shape[1]
         P = np.zeros(V.shape)
         P[:, :-1] = V[:, :-1] - V[:, 1:]
@@ -280,7 +292,7 @@ class ShenDirichletBasis(LaguerreBase):
 
     def to_ortho(self, input_array, output_array=None):
         if output_array is None:
-            output_array = Function(self.get_orthogonal())
+            output_array = np.zeros_like(input_array)
         else:
             output_array.fill(0)
         s0 = self.sl[slice(0, -1)]
@@ -313,24 +325,5 @@ class ShenDirichletBasis(LaguerreBase):
         output_array *= np.exp(-x/2)
         return output_array
 
-    def plan(self, shape, axis, dtype, options):
-        if isinstance(axis, tuple):
-            assert len(axis) == 1
-            axis = axis[0]
-
-        if isinstance(self.forward, Transform):
-            if self.forward.input_array.shape == shape and self.axis == axis:
-                # Already planned
-                return
-
-        self.LT.plan(shape, axis, dtype, options)
-        U, V = self.LT.forward.input_array, self.LT.forward.output_array
-        self.axis = axis
-        self.forward = Transform(self.forward, None, U, V, V)
-        self.backward = Transform(self.backward, None, V, V, U)
-        self.scalar_product = Transform(self.scalar_product, None, U, V, V)
-        self.si = islicedict(axis=self.axis, dimensions=self.dimensions)
-        self.sl = slicedict(axis=self.axis, dimensions=self.dimensions)
-
     def get_orthogonal(self):
-        return Basis(self.N, quad=self.quad, dtype=self.dtype, coordinates=self.coors.coordinates)
+        return Orthogonal(self.N, quad=self.quad, dtype=self.dtype, coordinates=self.coors.coordinates)
