@@ -4,11 +4,10 @@ import numpy as np
 import scipy.linalg as scipy_la
 from shenfun.optimization import optimizer
 from shenfun.optimization.cython import la
-from shenfun.utilities import inheritdocstrings
 from shenfun.la import TDMA as la_TDMA
 from shenfun.matrixbase import TPMatrix
 
-@inheritdocstrings
+
 class TDMA(la_TDMA):
     """Tridiagonal matrix solver
 
@@ -36,7 +35,7 @@ class TDMA(la_TDMA):
         return u
 
 
-class Helmholtz(object):
+class Helmholtz:
     r"""Helmholtz solver
 
     .. math::
@@ -129,8 +128,6 @@ class Helmholtz(object):
     """
 
     def __init__(self, *args, **kwargs):
-        local_shape = kwargs.get('local_shape', None)
-
         args = list(args)
         for i, arg in enumerate(args):
             if hasattr(arg, 'is_bc_matrix'):
@@ -164,10 +161,7 @@ class Helmholtz(object):
         self.axis = A.axis
         shape = [1]
         T = A.tensorproductspace
-        if local_shape is not None:
-            shape = list(local_shape)
-            shape[A.axis] = 1
-        elif T is not None:
+        if T is not None:
             shape = list(T.shape(True))
             shape[A.axis] = 1
 
@@ -255,7 +249,8 @@ class Helmholtz(object):
         c += c1
         return c
 
-class Biharmonic(object):
+
+class Biharmonic:
     r"""Multidimensional Biharmonic solver for
 
     .. math::
@@ -352,7 +347,7 @@ class Biharmonic(object):
         self.B = M['BBBmat']
 
         if len(args) == 3:
-            S_scale = np.atleast_1d(self.S.scale).item()
+            S_scale = self.S.scale
             A_scale = self.A.scale
             B_scale = self.B.scale
             if isinstance(self.S, TPMatrix):
@@ -429,7 +424,7 @@ class Biharmonic(object):
         c += c1
         return c
 
-class Helmholtz_2dirichlet(object):
+class Helmholtz_2dirichlet:
     """Helmholtz solver for 2-dimensional problems with 2 Dirichlet bases.
 
     .. math::
