@@ -31,7 +31,7 @@ as
 
 >>> import numpy as np
 >>> from shenfun.legendre.matrices import mat
->>> from shenfun.legendre.bases import ShenDirichletBasis as SD
+>>> from shenfun.legendre.bases import ShenDirichlet as SD
 >>> B = mat[((SD, 0), (SD, 0))]
 
 and an instance of the matrix can be created as
@@ -69,26 +69,26 @@ import functools
 import numpy as np
 import sympy as sp
 from shenfun.matrixbase import SpectralMatrix
-from shenfun.utilities import inheritdocstrings
 from shenfun.la import TDMA as neumann_TDMA
+from shenfun.optimization import cython
 from .la import TDMA
 from . import bases
 
 # Short names for instances of bases
-LB = bases.Basis
-SD = bases.ShenDirichletBasis
-SB = bases.ShenBiharmonicBasis
-SN = bases.ShenNeumannBasis
-SU = bases.UpperDirichletBasis
-CD = bases.BCBasis
-CB = bases.BCBiharmonicBasis
+LB = bases.Orthogonal
+SD = bases.ShenDirichlet
+SB = bases.ShenBiharmonic
+SN = bases.ShenNeumann
+SU = bases.UpperDirichlet
+CD = bases.BCDirichlet
+CB = bases.BCBiharmonic
 
 x = sp.symbols('x', real=True)
 xp = sp.symbols('x', real=True, positive=True)
 
 #pylint: disable=unused-variable, redefined-builtin, bad-continuation
 
-@inheritdocstrings
+
 class BLLmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -130,7 +130,6 @@ class BLLmat(SpectralMatrix):
         return u
 
 
-@inheritdocstrings
 class BDDmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -167,7 +166,6 @@ class BDDmat(SpectralMatrix):
         self.solve = TDMA(self)
 
 
-@inheritdocstrings
 class BNNmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -200,7 +198,6 @@ class BNNmat(SpectralMatrix):
         #self.solve = neumann_TDMA(self)
 
 
-@inheritdocstrings
 class BBBmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -236,7 +233,7 @@ class BBBmat(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
         self.solve = PDMA(self)
 
-@inheritdocstrings
+
 class BDLmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -267,7 +264,7 @@ class BDLmat(SpectralMatrix):
             d[2][-1] = -2./(N-1)*sc[N-3]
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
-@inheritdocstrings
+
 class BLDmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -299,7 +296,7 @@ class BLDmat(SpectralMatrix):
             d[-2][-1] = -2./(N-1)*sc[-3]
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
-@inheritdocstrings
+
 class ADDmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
 
@@ -369,7 +366,6 @@ class ADDmat(SpectralMatrix):
         return u
 
 
-@inheritdocstrings
 class ANNmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
 
@@ -428,7 +424,6 @@ class ANNmat(SpectralMatrix):
         return u
 
 
-@inheritdocstrings
 class ABBmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
 
@@ -456,7 +451,6 @@ class ABBmat(SpectralMatrix):
         d[-2] = d[2]
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
-@inheritdocstrings
 class GLLmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
 
@@ -481,11 +475,26 @@ class GLLmat(SpectralMatrix):
         d = {}
         for j in range(2, N, 2):
             jj = j if trial[1] else -j
-            d[jj] = (k[:-j]+0.5)*((k[:-j]+j)*(k[:-j]+j+1) - k[:-j]*(k[:-j]+1))*2./(2*k[:-j]+1)
+            d[jj] = (k[:-j]+0.5)*(k[j:]*(k[j:]+1) - k[:-j]*(k[:-j]+1))*2./(2*k[:-j]+1)
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        self._matvec_methods += ['cython']
 
+    def matvec(self, v, c, format='cython', axis=0):
+        c.fill(0)
+        trial = self.trialfunction[1]
+        if format == 'cython' and v.ndim == 3 and trial:
+            cython.Matvec.GLL_matvec3D_ptr(v, c, axis)
+            self.scale_array(c)
+        elif format == 'cython' and v.ndim == 2 and trial:
+            cython.Matvec.GLL_matvec2D_ptr(v, c, axis)
+            self.scale_array(c)
+        elif format == 'cython' and v.ndim == 1 and trial:
+            cython.Matvec.GLL_matvec(v, c)
+            self.scale_array(c)
+        else:
+            c = super(GLLmat, self).matvec(v, c, format=format, axis=axis)
+        return c
 
-@inheritdocstrings
 class SBBmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
 
@@ -510,7 +519,7 @@ class SBBmat(SpectralMatrix):
         d = {0: 2*(2*k+3)**2*(2*k+5)}
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
-@inheritdocstrings
+
 class CLLmat(SpectralMatrix):
     r"""Matrix for inner product
 
@@ -524,7 +533,7 @@ class CLLmat(SpectralMatrix):
 
         j = 0, 1, ..., N \text{ and } k = 0, 1, ..., N
 
-    and :math:`\psi_k` is the Shen Legendre basis function.
+    and :math:`\psi_k` is the orthogonal Legendre basis function.
 
     """
     def __init__(self, test, trial, measure=1):
@@ -533,8 +542,9 @@ class CLLmat(SpectralMatrix):
         N = test[0].N
         d = {}
         for i in range(1, N, 2):
-            d[i] = 2.0
+            d[i] = 2
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+        self._matvec_methods += ['cython', 'self']
 
     def matvec(self, v, c, format='self', axis=0):
         c.fill(0)
@@ -542,23 +552,28 @@ class CLLmat(SpectralMatrix):
             if axis > 0:
                 c = np.moveaxis(c, axis, 0)
                 v = np.moveaxis(v, axis, 0)
-            #s = (slice(None),) + (np.newaxis,)*(v.ndim-1) # broadcasting
             ve = v[-2:0:-2].cumsum(axis=0)
             vo = v[-1:0:-2].cumsum(axis=0)
-            c[-3::-2] = ve*2.0
-            c[-2::-2] = vo*2.0
+            c[-3::-2] = ve*2
+            c[-2::-2] = vo*2
             if axis > 0:
                 c = np.moveaxis(c, 0, axis)
                 v = np.moveaxis(v, 0, axis)
-            c *= self.scale
+            self.scale_array(c)
 
+        elif format == 'cython' and v.ndim == 3:
+            cython.Matvec.CLL_matvec3D_ptr(v, c, axis)
+            self.scale_array(c)
+        elif format == 'cython' and v.ndim == 2:
+            cython.Matvec.CLL_matvec2D_ptr(v, c, axis)
+            self.scale_array(c)
+        elif format == 'cython' and v.ndim == 1:
+            cython.Matvec.CLL_matvec(v, c)
+            self.scale_array(c)
         else:
             c = super(CLLmat, self).matvec(v, c, format=format, axis=axis)
-
         return c
 
-
-@inheritdocstrings
 class CLLmatT(SpectralMatrix):
     r"""Matrix for inner product
 
@@ -572,7 +587,7 @@ class CLLmatT(SpectralMatrix):
 
         j = 0, 1, ..., N \text{ and } k = 0, 1, ..., N
 
-    and :math:`\psi_k` is the Shen Legendre basis function.
+    and :math:`\psi_k` is the orthogonal Legendre basis function.
 
     """
     def __init__(self, test, trial, measure=1):
@@ -581,10 +596,10 @@ class CLLmatT(SpectralMatrix):
         N = test[0].N
         d = {}
         for i in range(-1, -N, -2):
-            d[i] = 2.0
+            d[i] = 2
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
-@inheritdocstrings
+
 class CLDmat(SpectralMatrix):
     r"""Matrix for inner product
 
@@ -611,7 +626,7 @@ class CLDmat(SpectralMatrix):
             d[-1] = -2. / np.sqrt(4*k+6)
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
-@inheritdocstrings
+
 class CDLmat(SpectralMatrix):
     r"""Matrix for inner product
 
@@ -638,7 +653,7 @@ class CDLmat(SpectralMatrix):
             d[1] = -2. / np.sqrt(4*k+6)
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
-@inheritdocstrings
+
 class CDDmat(SpectralMatrix):
     r"""Matrix for inner product
 
@@ -666,6 +681,7 @@ class CDDmat(SpectralMatrix):
             d[1] = 2. / np.sqrt(4*k[:-1]+6)
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
+
 class ADDrp1mat(SpectralMatrix):
     r"""Matrix for inner product
 
@@ -691,6 +707,7 @@ class ADDrp1mat(SpectralMatrix):
         d = {0: 4*k+6, 1: 2*k[:-1]+4, -1: 2*k[:-1]+4}
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
+
 class ADD2rp1mat(SpectralMatrix):
     r"""Matrix for inner product
 
@@ -716,6 +733,7 @@ class ADD2rp1mat(SpectralMatrix):
         d = {0: -(4*k+6), 1: -(2*k[:-1]+6), -1: -(2*k[:-1]+2)}
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
+
 class ADD2Trp1mat(SpectralMatrix):
     r"""Matrix for inner product
 
@@ -741,6 +759,7 @@ class ADD2Trp1mat(SpectralMatrix):
         d = {0: -(4*k+6), -1: -(2*k[:-1]+6), 1: -(2*k[:-1]+2)}
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
+
 class AUUrp1mat(SpectralMatrix):
     r"""Matrix for inner product
 
@@ -765,6 +784,7 @@ class AUUrp1mat(SpectralMatrix):
         d = {0: 2*k+2}
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
+
 class BUUrp1mat(SpectralMatrix):
     r"""Matrix for inner product
 
@@ -820,6 +840,7 @@ class BDD1orp1mat(SpectralMatrix):
         d = {0: 2*(2*k+3)/(k+1)/(k+2), 1: -2/(k[:-1]+2), -1: -2/(k[:-1]+2)}
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
+
 class BDDrp1mat(SpectralMatrix):
     r"""Matrix for inner product
 
@@ -852,7 +873,6 @@ class BDDrp1mat(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
-@inheritdocstrings
 class BCDmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -886,7 +906,7 @@ class BCDmat(SpectralMatrix):
 
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
-@inheritdocstrings
+
 class BCBmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
@@ -919,7 +939,7 @@ class BCBmat(SpectralMatrix):
 
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
-@inheritdocstrings
+
 class _Legmatrix(SpectralMatrix):
     def __init__(self, test, trial, measure=1):
         SpectralMatrix.__init__(self, {}, test, trial, measure=measure)
