@@ -1,144 +1,6 @@
 r"""
 This module contains classes for working with sparse matrices.
 
-The sparse matrices are computed as inner products of forms containing test and
-trial functions. These basis functions are chosen from the following, where we
-denote the :math:`k`'th basis function of basis V as :math:`\phi_k`:
-
-Chebyshev basis:
-    Chebyshev basis of first kind
-
-    .. math::
-
-        \phi_k &= T_k \\
-        V &= span\{\phi_k\}_{k=0}^{N}
-
-    For homogeneous Dirichlet boundary conditions:
-
-    .. math::
-
-        \phi_k &= T_k - T_{k+2} \\
-        V &= span\{\phi_k\}_{k=0}^{N-2}
-
-    For homogeneous Neumann boundary conditions:
-
-    .. math::
-
-        \phi_k &= T_k - \left(\frac{k}{k+2}\right)^2T_{k+2} \\
-        V &= span\{\phi_k\}_{k=1}^{N-2}
-
-    For Biharmonic basis with both homogeneous Dirichlet and Neumann:
-
-    .. math::
-
-        \phi_k &= T_k - 2 \frac{k+2}{k+3} T_{k+2} + \frac{k+1}{k+3} T_{k+4} \\
-        V &= span\{\phi_k\}_{k=0}^{N-4}
-
-    The scalar product is computed as a weighted inner product with
-
-    :math:`w=1/\sqrt{1-x^2}` the weights.
-
-Legendre basis:
-    Regular Legendre
-
-    .. math::
-
-        \phi_k &= L_k \\
-        V &= span\{\phi_k\}_{k=0}^{N}
-
-    Dirichlet boundary conditions
-
-    .. math::
-
-        \phi_k &= L_k-L_{k+2} \\
-        V &= span\{\phi_k\}_{k=0}^{N-2}
-
-    Homogeneous Neumann boundary conditions:
-
-    .. math::
-
-        \phi_k &= L_k - \frac{k(k+1)}{(k+2)(k+3)}L_{k+2} \\
-        V &= span\{\phi_k\}_{k=1}^{N-2}
-
-    Both homogeneous Dirichlet and Neumann:
-
-    .. math::
-
-        \psi_k &= L_k - 2 \frac{2k+5}{2k+7} L_{k+2} + \frac{2k+3}{2k+7} L_{k+4} \\
-        V &= span\{\phi_k\}_{k=0}^{N-4}
-
-Laguerre basis:
-    Regular Laguerre function
-
-    .. math::
-
-        \phi_k &= L_k \cdot \exp(-x) \\
-        V &= span\{\phi_k\}_{k=0}^{N}
-
-    Homogeneous Dirichlet boundary conditions
-
-    .. math::
-
-        \phi_k &= (L_k-L_{k+2}) \cdot \exp(-x) \\
-        V &= span\{\phi_k\}_{k=0}^{N-1}
-
-    where :math:`L_k` is the Laguerre polynomial of order k.
-
-Hermite basis:
-    Regular Hermite function
-
-    .. math::
-
-        \phi_k &= H_k \cdot \exp(-x^2/2)/(\pi^{0.25}\sqrt{2^k k!}) \\
-        V &= span\{\phi_k\}_{k=0}^{N}
-
-    where :math:`K_k` is the Hermite polynomial of order k.
-    Homogeneous Dirichlet boundary conditions on (-inf, inf)
-
-Fourier basis:
-
-    .. math::
-
-        \phi_k &= exp(ikx) \\
-        V &= span\{\phi_k\}_{k=-N/2}^{N/2-1}
-
-Jacobi basis:
-
-    Regular Jacobi polynomials
-
-    .. math::
-
-        \phi_k(x) &= J_k(x, \alpha, \beta) \\
-        V &= span\{\phi_k\}_{k=0}^{N-1}
-
-    where :math:`\alpha > -1` and :math:`\beta > -1`. :math:`J_k` is
-    the regular Jacobi polynomial
-
-    Homogeneous Dirichlet boundary conditions
-
-    .. math::
-
-        \phi_k &= j_k(x, -1, -1)
-        V &= span\{\phi_k\}_{k=0}^{N-3}
-
-    where :math:`j_k` is the generalized Jacobi polynomial
-
-    Homogeneous Dirichlet and Neumann boundary conditions
-
-    .. math::
-
-        \phi_k &= j_k(x, -2, -2)
-        V &= span\{\phi_k\}_{k=0}^{N-5}
-
-    Homogeneous Dirichlet and first and second order derivatives
-    (for 6th order equation)
-
-    .. math::
-
-        \phi_k &= j_k(x, -3, -3)
-        V &= span\{\phi_k\}_{k=0}^{N-7}
-
-
 """
 from __future__ import division
 from copy import deepcopy
@@ -148,7 +10,7 @@ import sympy as sp
 from scipy.sparse import bmat, dia_matrix, kron, diags as sp_diags
 from scipy.sparse.linalg import spsolve
 from mpi4py import MPI
-from .utilities import inheritdocstrings, integrate_sympy
+from .utilities import integrate_sympy
 
 __all__ = ['SparseMatrix', 'SpectralMatrix', 'extract_diagonal_matrix',
            'check_sanity', 'get_dense_matrix', 'TPMatrix', 'BlockMatrix',
@@ -199,6 +61,7 @@ class SparseMatrix(dict):
         self.shape = shape
         self._diags = dia_matrix((1, 1))
         self.scale = scale
+        self._matvec_methods = []
 
     def matvec(self, v, c, format='dia', axis=0):
         """Matrix vector product
@@ -454,17 +317,11 @@ class SparseMatrix(dict):
         return self.__hash__() == a.__hash__()
 
     def scale_array(self, c):
-        if isinstance(self.scale, Number):
-            if self.scale not in (1.0, 1):
-                c *= self.scale
-        else:
-            if np.prod(self.scale.shape) == 1: # array with only one number
-                if self.scale not in (1.0, 1):
-                    c *= self.scale
-            else:
-                c *= self.scale
+        assert isinstance(self.scale, Number)
+        if abs(self.scale-1) > 1e-8:
+            c *= self.scale
 
-    def solve(self, b, u=None, axis=0):
+    def solve(self, b, u=None, axis=0, use_lu=False):
         """Solve matrix system Au = b
 
         where A is the current matrix (self)
@@ -479,6 +336,8 @@ class SparseMatrix(dict):
         axis : int, optional
             The axis over which to solve for if b and u are multi-
             dimensional
+        use_lu : bool, optional
+            Look for already computed LU-matrix
 
         Note
         ----
@@ -500,11 +359,25 @@ class SparseMatrix(dict):
                 b = np.moveaxis(b, axis, 0)
 
         if b.ndim == 1:
-            u[:] = spsolve(self.diags('csr'), b)
+            if use_lu:
+                if b.dtype.char in 'FDG':
+                    u.real[:] = self._lu.solve(b.real)
+                    u.imag[:] = self._lu.solve(b.imag)
+                else:
+                    u[:] = self._lu.solve(b)
+            else:
+                u[:] = spsolve(self.diags('csc'), b)
         else:
             N = b.shape[0]
             P = np.prod(b.shape[1:])
-            u[:] = spsolve(self.diags('csr'), b.reshape((N, P))).reshape(u.shape)
+            if use_lu:
+                if b.dtype.char in 'FDG':
+                    u.real[:] = self._lu.solve(b.real.reshape((N, P))).reshape(u.shape)
+                    u.imag[:] = self._lu.solve(b.imag.reshape((N, P))).reshape(u.shape)
+                else:
+                    u[:] = self._lu.solve(b.reshape((N, P))).reshape(u.shape)
+            else:
+                u[:] = spsolve(self.diags('csc'), b.reshape((N, P))).reshape(u.shape)
 
         if axis > 0:
             u = np.moveaxis(u, 0, axis)
@@ -514,13 +387,14 @@ class SparseMatrix(dict):
 
     def isdiagonal(self):
         if len(self) == 1:
-            return True
+            if (0 in self):
+                return True
         return False
 
     def isidentity(self):
         if not len(self) == 1:
             return False
-        if 0 not in self:
+        if (0 not in self):
             return False
         d = self[0]
         if np.all(d == 1):
@@ -546,7 +420,7 @@ class SparseMatrix(dict):
         return self
 
 
-@inheritdocstrings
+
 class SpectralMatrix(SparseMatrix):
     r"""Base class for inner product matrices.
 
@@ -590,7 +464,7 @@ class SpectralMatrix(SparseMatrix):
     The matrices can be automatically created using, e.g., for the mass
     matrix of the Dirichlet space::
 
-        SD = ShenDirichletBasis
+        SD = ShenDirichlet
         N = 16
         M = SpectralMatrix({}, (SD(N), 0), (SD(N), 0))
 
@@ -620,6 +494,7 @@ class SpectralMatrix(SparseMatrix):
         shape = (test[0].dim(), trial[0].dim())
         if d == {}:
             D = get_dense_matrix(test, trial, measure)[:shape[0], :shape[1]]
+            #D = get_dense_matrix_sympy(test, trial, measure)[:shape[0], :shape[1]]
             d = extract_diagonal_matrix(D)
         SparseMatrix.__init__(self, d, shape, scale)
         if shape[0] == shape[1]:
@@ -631,12 +506,12 @@ class SpectralMatrix(SparseMatrix):
         ss = [slice(None)]*len(v.shape)
         ss[axis] = u.slice()
         c = super(SpectralMatrix, self).matvec(v[tuple(ss)], c, format=format, axis=axis)
-        if self.testfunction[0].__class__.__name__ == 'ShenNeumannBasis':
+        if self.testfunction[0].__class__.__name__ == 'ShenNeumann':
             ss[axis] = 0
             c[tuple(ss)] = 0
         return c
 
-    def solve(self, b, u=None, axis=0):
+    def solve(self, b, u=None, axis=0, use_lu=False):
         """Solve matrix system Au = b
 
         where A is the current matrix (self)
@@ -650,13 +525,15 @@ class SpectralMatrix(SparseMatrix):
             Output array
         axis : int, optional
                The axis over which to solve for if b and u are multidimensional
+        use_lu : bool, optional
+            Look for already computed LU-matrix
 
         Note
         ----
         Vectors may be one- or multidimensional.
 
         """
-        u = self.solver(b, u=u, axis=axis)
+        u = self.solver(b, u=u, axis=axis, use_lu=use_lu)
         return u
 
     @property
@@ -680,8 +557,8 @@ class SpectralMatrix(SparseMatrix):
             return  self.__class__.__name__
         return self.__hash__()
 
-    def simplify_fourier_matrices(self):
-        if self.testfunction[0].family() == 'fourier':
+    def simplify_diagonal_matrices(self):
+        if self.isdiagonal():
             self.scale = self.scale*self[0]
             self[0] = 1
 
@@ -690,6 +567,10 @@ class SpectralMatrix(SparseMatrix):
             return False
         if self.get_key() != a.get_key():
             return False
+        if not np.array(list(self.keys())).shape == np.array(list(a.keys())).shape:
+            return False
+        #if not np.all(np.array(list(self.keys())) == np.array(list(a.keys()))):
+        #    return False
         if np.linalg.norm(self[list(self.keys())[0]] - a[list(self.keys())[0]]) > 1e-8:
             return False
         return True
@@ -717,7 +598,6 @@ class SpectralMatrix(SparseMatrix):
     def __add__(self, y):
         """Return copy of self.__add__(y) <==> self+y"""
         assert isinstance(y, dict)
-        # Check is the same matrix
         if self == y:
             f = SpectralMatrix(deepcopy(dict(self)), self.testfunction,
                                self.trialfunction, self.scale+y.scale)
@@ -736,7 +616,6 @@ class SpectralMatrix(SparseMatrix):
     def __sub__(self, y):
         """Return copy of self.__sub__(y) <==> self-y"""
         assert isinstance(y, dict)
-        # Check is the same matrix
         if self == y:
             f = SpectralMatrix(deepcopy(dict(self)), self.testfunction,
                                self.trialfunction, self.scale-y.scale)
@@ -767,7 +646,17 @@ class Identity(SparseMatrix):
     def __init__(self, shape, scale=1):
         SparseMatrix.__init__(self, {0:1}, shape, scale)
 
-class BlockMatrix(object):
+    def solve(self, b, u=None, axis=0):
+        if u is None:
+            u = b
+        else:
+            assert u.shape == b.shape
+            u[:] = b
+        u *= (1/self.scale)
+        return u
+
+
+class BlockMatrix:
     r"""A class for block matrices
 
     Parameters
@@ -820,14 +709,14 @@ class BlockMatrix(object):
     >>> from mpi4py import MPI
     >>> comm = MPI.COMM_WORLD
     >>> N = (24, 24, 24)
-    >>> K0 = Basis(N[0], 'Fourier', dtype='d')
-    >>> K1 = Basis(N[1], 'Fourier', dtype='D')
-    >>> SD = Basis(N[2], 'Legendre', bc=(0, 0))
-    >>> ST = Basis(N[2], 'Legendre')
+    >>> K0 = FunctionSpace(N[0], 'Fourier', dtype='d')
+    >>> K1 = FunctionSpace(N[1], 'Fourier', dtype='D')
+    >>> SD = FunctionSpace(N[2], 'Legendre', bc=(0, 0))
+    >>> ST = FunctionSpace(N[2], 'Legendre')
     >>> TD = TensorProductSpace(comm, (K0, K1, SD), axes=(2, 1, 0))
     >>> TT = TensorProductSpace(comm, (K0, K1, ST), axes=(2, 1, 0))
-    >>> VT = VectorTensorProductSpace(TD)
-    >>> Q = MixedTensorProductSpace([VT, TD])
+    >>> VT = VectorSpace(TD)
+    >>> Q = CompositeSpace([VT, TD])
     >>> up = TrialFunction(Q)
     >>> vq = TestFunction(Q)
     >>> u, p = up
@@ -844,7 +733,7 @@ class BlockMatrix(object):
         self.mixedbase = mixedbase = tpmats[0][0].mixedbase
         self.dims = dims = mixedbase.num_components()
         self.mats = np.zeros((dims, dims), dtype=int).tolist()
-        tps = mixedbase.flatten()
+        tps = mixedbase.flatten() if hasattr(mixedbase, 'flatten') else [mixedbase]
         offset = [np.zeros(tps[0].dimensions, dtype=int)]
         for i, tp in enumerate(tps):
             dims = tp.dim() if not hasattr(tp, 'dims') else tp.dims() # 1D basis does not have dims()
@@ -955,7 +844,7 @@ class BlockMatrix(object):
             The format of the returned matrix. See `Scipy sparse matrices <https://docs.scipy.org/doc/scipy/reference/sparse.html>`_
 
         """
-        from .spectralbase import MixedBasis
+        from .spectralbase import MixedFunctionSpace
         bm = []
         for mi in self.mats:
             bm.append([])
@@ -964,7 +853,7 @@ class BlockMatrix(object):
                     bm[-1].append(None)
                 else:
                     m = mij[0]
-                    if isinstance(self.mixedbase, MixedBasis):
+                    if isinstance(self.mixedbase, MixedFunctionSpace):
                         d = m.diags(format)
                         for mj in mij[1:]:
                             d = d + mj.diags(format)
@@ -1053,7 +942,6 @@ class BlockMatrix(object):
             assert isinstance(con[0], Integral)
             assert isinstance(con[1], Integral)
             assert isinstance(con[2], Number)
-            #assert tp[con[0]].bases[axis].boundary_condition().lower() in ('', 'periodic')
         N = self.global_shape[axis]
         gi = np.zeros(N, dtype=b.dtype)
         go = np.zeros(N, dtype=b.dtype)
@@ -1184,7 +1072,7 @@ class BlockMatrix(object):
 
         return A, b
 
-class TPMatrix(object):
+class TPMatrix:
     """Tensorproduct matrix
 
     A :class:`.TensorProductSpace` is the outer product of ``D`` bases.
@@ -1198,34 +1086,37 @@ class TPMatrix(object):
         Instances of :class:`.SpectralMatrix` or :class:`.SparseMatrix`
         The length of ``mats`` is the number of dimensions of the
         :class:`.TensorProductSpace`
-    space : Function space
+    testspace : Function space
         The test :class:`.TensorProductSpace`
+    trialspace : Function space
+        The trial :class:`.TensorProductSpace`
     scale : array, optional
         Scalar multiple of matrices. Must have ndim equal to the number of
         dimensions in the :class:`.TensorProductSpace`, and the shape must be 1
         along any directions with a nondiagonal matrix.
     global_index : 2-tuple, optional
-        Indices (test, trial) into mixed space :class:`.MixedTensorProductSpace`.
-    mixedbase : :class:`.MixedTensorProductSpace`, optional
+        Indices (test, trial) into mixed space :class:`.CompositeSpace`.
+    mixedbase : :class:`.CompositeSpace`, optional
          Instance of the base space
     """
-    def __init__(self, mats, space, scale=1.0, global_index=None, mixedbase=None):
+    def __init__(self, mats, testspace, trialspace, scale=1.0, global_index=None, mixedbase=None):
         assert isinstance(mats, (list, tuple))
-        assert len(mats) == len(space)
+        assert len(mats) == len(testspace)
         self.mats = mats
-        self.space = space
+        self.space = testspace
+        self.trialspace = trialspace
         self.scale = scale
         self.pmat = 1
         self.naxes = []
         self.global_index = global_index
         self.mixedbase = mixedbase
 
-    def simplify_fourier_matrices(self):
+    def simplify_diagonal_matrices(self):
         self.naxes = []
         for axis, mat in enumerate(self.mats):
             if not mat:
                 continue
-            if self.space[axis].family() == 'fourier':
+            if axis not in self.space.get_nondiagonal_axes():
                 if self.dimensions == 1: # Don't bother with the 1D case
                     continue
                 else:
@@ -1235,6 +1126,7 @@ class TPMatrix(object):
                         d *= mat.scale
                     self.scale = self.scale*d
                     self.mats[axis] = Identity(mat.shape)
+
             else:
                 self.naxes.append(axis)
         # Decomposition
@@ -1252,20 +1144,23 @@ class TPMatrix(object):
         if len(self.naxes) == 1:
             self.pmat = self.mats[self.naxes[0]]
 
-        elif len(self.naxes) == 2: # 2 nonperiodic, experimental
+        elif len(self.naxes) == 2: # 2 nondiagonal
             self.pmat = self.mats
 
     def solve(self, b, u=None):
+
         if len(self.naxes) == 0:
+            sl = tuple([s.slice() for s in self.trialspace.bases])
             d = self.scale
             with np.errstate(divide='ignore'):
                 d = 1./self.scale
             d = np.where(np.isfinite(d), d, 0)
+            if u is None:
+                from .forms.arguments import Function
+                u = Function(self.space)
 
-            if u is not None:
-                u[:] = b * d
-                return u
-            return b * d
+            u[sl] = b[sl] * d[sl]
+            return u
 
         elif len(self.naxes) == 1:
             axis = self.naxes[0]
@@ -1276,8 +1171,8 @@ class TPMatrix(object):
             return u
 
         elif len(self.naxes) == 2:
-            from shenfun.la import SolverGeneric2NP
-            H = SolverGeneric2NP([self])
+            from shenfun.la import SolverGeneric2ND
+            H = SolverGeneric2ND([self])
             u = H(b, u)
             return u
 
@@ -1317,8 +1212,16 @@ class TPMatrix(object):
         return c
 
     def get_key(self):
-        assert len(self.naxes) == 1
-        return self.pmat.get_key()
+        """Return key of the one nondiagonal matrix in the TPMatrix
+
+        Note
+        ----
+        Raises an error of there are more than one single nondiagonal matrix
+        in TPMatrix.
+        """
+        naxis = self.space.get_nondiagonal_axes()
+        assert len(naxis) == 1
+        return self.mats[naxis[0]].get_key()
 
     def isidentity(self):
         return np.all([m.isidentity() for m in self.mats])
@@ -1341,7 +1244,7 @@ class TPMatrix(object):
     def __mul__(self, a):
         """Returns copy of self.__mul__(a) <==> self*a"""
         if isinstance(a, Number):
-            TPMatrix(self.mats, self.space, self.scale*a,
+            TPMatrix(self.mats, self.space, self.trialspace, self.scale*a,
                      self.global_index, self.mixedbase)
 
         elif isinstance(a, np.ndarray):
@@ -1367,7 +1270,7 @@ class TPMatrix(object):
     def __div__(self, a):
         """Returns copy self.__div__(a) <==> self/a"""
         if isinstance(a, Number):
-            return TPMatrix(self.mats, self.space, self.scale/a,
+            return TPMatrix(self.mats, self.space, self.trialspace, self.scale/a,
                             self.global_index, self.mixedbase)
         elif isinstance(a, np.ndarray):
             b = np.zeros_like(a)
@@ -1405,7 +1308,7 @@ class TPMatrix(object):
         """Return copy of self.__add__(a) <==> self+a"""
         assert isinstance(a, TPMatrix)
         assert self == a
-        return TPMatrix(self.mats, self.space, self.scale+a.scale,
+        return TPMatrix(self.mats, self.space, self.trialspace, self.scale+a.scale,
                         self.global_index, self.mixedbase)
 
     def __iadd__(self, a):
@@ -1419,7 +1322,7 @@ class TPMatrix(object):
         """Return copy of self.__sub__(a) <==> self-a"""
         assert isinstance(a, TPMatrix)
         assert self == a
-        return TPMatrix(self.mats, self.space, self.scale-a.scale,
+        return TPMatrix(self.mats, self.space, self.trialspace, self.scale-a.scale,
                         self.global_index, self.mixedbase)
 
     def __isub__(self, a):
@@ -1429,6 +1332,12 @@ class TPMatrix(object):
         self.scale = self.scale - a.scale
         return self
 
+    def diags(self, format='csr'):
+        if self.dimensions == 2:
+            return kron(self.mats[0].diags(format=format), self.mats[1].diags(format=format))
+        elif self.dimensions == 3:
+            return kron(self.mats[0].diags(format=format), kron(self.mats[1].diags(format=format), self.mats[2].diags(format=format)))
+
 
 def check_sanity(A, test, trial, measure=1):
     """Sanity check for matrix.
@@ -1481,16 +1390,16 @@ def get_dense_matrix(test, trial, measure=1):
         As test, but representing matrix column.
     measure : Sympy expression of coordinate, or number, optional
         Additional weight to integral. For example, in cylindrical
-        coordinates an additional measure is the readius `r`.
+        coordinates an additional measure is the radius `r`.
     """
     N = test[0].N
     x = test[0].mpmath_points_and_weights(N, map_true_domain=False)[0]
     ws = test[0].get_measured_weights(N, measure)
     v = test[0].evaluate_basis_derivative_all(x=x, k=test[1])
     u = trial[0].evaluate_basis_derivative_all(x=x, k=trial[1])
-    return np.dot(v.T*ws[np.newaxis, :], np.conj(u))
+    return np.dot(np.conj(v.T)*ws[np.newaxis, :], u)
 
-def extract_diagonal_matrix(M, abstol=1e-8, reltol=1e-8):
+def extract_diagonal_matrix(M, abstol=1e-14, reltol=1e-12):
     """Return SparseMatrix version of dense matrix ``M``
 
     Parameters
@@ -1521,7 +1430,7 @@ def extract_diagonal_matrix(M, abstol=1e-8, reltol=1e-8):
 
     return SparseMatrix(d, M.shape)
 
-def get_dense_matrix_sympy(test, trial, measure=1, x=sp.symbols('x', real=True)):
+def get_dense_matrix_sympy(test, trial, measure=1):
     """Return dense matrix automatically computed from basis
 
     Parameters
@@ -1543,22 +1452,31 @@ def get_dense_matrix_sympy(test, trial, measure=1, x=sp.symbols('x', real=True))
     measure : Sympy expression of coordinate, or number, optional
         Additional weight to integral. For example, in cylindrical
         coordinates an additional measure is the radius `r`.
-    x : Sympy Symbol, optional
     """
     N = test[0].slice().stop - test[0].slice().start
     M = trial[0].slice().stop - trial[0].slice().start
-    V = np.zeros((N, M))
+    V = np.zeros((N, M), dtype=test[0].forward.output_array.dtype)
+    x = sp.Symbol('x', real=True)
 
     if not measure == 1:
         if isinstance(measure, sp.Expr):
+            s = measure.free_symbols
+            assert len(s) == 1
+            x = s.pop()
             xm = test[0].map_true_domain(x)
             measure = measure.subs(x, xm)
         else:
             assert isinstance(measure, Number)
+
+    # Weight of weighted space
+    measure *= test[0].weight()
+
     for i in range(test[0].slice().start, test[0].slice().stop):
-        pi = test[0].sympy_basis(i, x=x)
+        pi = np.conj(test[0].sympy_basis(i, x=x))
         for j in range(trial[0].slice().start, trial[0].slice().stop):
             pj = trial[0].sympy_basis(j, x=x)
-            V[i, j] = integrate_sympy(sp.simplify(measure*pi.diff(x, test[1])*pj.diff(x, trial[1])),
-                                      (x, test[0].reference_domain()[0], test[0].reference_domain()[1]))
+            integrand = sp.simplify(measure*pi.diff(x, test[1])*pj.diff(x, trial[1]))
+            V[i, j] = integrate_sympy(integrand,
+                                      (x, test[0].sympy_reference_domain()[0], test[0].sympy_reference_domain()[1]))
+
     return V
