@@ -6,7 +6,7 @@ from numbers import Number
 import functools
 import sympy as sp
 import numpy as np
-from shenfun.fourier.bases import R2CBasis, C2CBasis
+from shenfun.fourier.bases import R2C, C2C
 from shenfun.utilities import apply_mask
 from shenfun.forms.arguments import Function, Array
 from shenfun.optimization.cython import evaluate
@@ -14,9 +14,12 @@ from shenfun.spectralbase import slicedict, islicedict, SpectralBase
 from shenfun.coordinates import Coordinates
 from mpi4py_fft.mpifft import Transform, PFFT
 from mpi4py_fft.pencil import Subcomm, Pencil
+from mpi4py import MPI
 
-__all__ = ('TensorProductSpace', 'VectorTensorProductSpace',
-           'MixedTensorProductSpace', 'Convolve')
+comm = MPI.COMM_WORLD
+
+__all__ = ('TensorProductSpace', 'VectorSpace', 'TensorSpace',
+           'CompositeSpace', 'Convolve')
 
 # Default sympy symbols. Note that order is important
 x, y, z, r, s = psi = sp.symbols('x,y,z,r,s', real=True)
@@ -43,9 +46,9 @@ class CurvilinearTransform(Transform):
         self._T = T
 
     @property
-    def hi(self):
+    def sg(self):
         """Return scaling factors of Transform"""
-        return self._T.hi
+        return self._T.coors.get_sqrt_det_g()
 
     @property
     def local_mesh(self):
@@ -55,7 +58,7 @@ class CurvilinearTransform(Transform):
     def get_measured_input_array(self):
         """Weigh input array with integral measure
         """
-        dx = self.hi.prod()
+        dx = self.sg
         mesh = self.local_mesh
         if dx == 1:
             return
@@ -133,7 +136,7 @@ class TensorProductSpace(PFFT):
         Pencil distribution in spectral space. This is primarily intended
         for a padded space, where the spectral distribution must be
         equal to the non-padded space.
-    coordinates: two tuples, optional
+    coordinates : 2- or 3-tuple, optional
         Map for curvilinear coordinatesystem.
         First tuple are the coordinate variables in the new coordinate system
         Second tuple are the Cartesian coordinates as functions of the variables
@@ -143,7 +146,16 @@ class TensorProductSpace(PFFT):
             rv = (r*sp.cos(theta), r*sp.sin(theta))
 
         where psi and rv are the first and second tuples, respectively.
-
+        If a third item is provided with the tuple, then this third item
+        is used as an additional assumption. For example, it is necessary
+        to provide the assumption `sympy.Q.positive(sympy.sin(theta))`, such
+        that sympy can evaluate `sqrt(sympy.sin(theta)**2)` to `sympy.sin(theta)`
+        and not `Abs(sympy.sin(theta))`. Different coordinates may require
+        different assumptions to help sympy when computing basis functions
+        etc.
+    modify_spaces_inplace : bool, optional
+        Whether or not a copy should be made of the input functionspaces.
+        If True, then the input spaces will be modified inplace.
     kw : dict, optional
         Dictionary that can be used to plan transforms. Input to method
         `plan` for the bases.
@@ -151,17 +163,25 @@ class TensorProductSpace(PFFT):
     """
     def __init__(self, comm, bases, axes=None, dtype=None, slab=False,
                  collapse_fourier=False, backward_from_pencil=False,
-                 coordinates=None, **kw):
+                 coordinates=None, modify_spaces_inplace=False, **kw):
         # Note do not call __init__ of super
         self.comm = comm
         self.bases = bases
+        if not modify_spaces_inplace:
+            self.bases = tuple([base.get_unplanned() for base in bases])
+
         coors = coordinates if coordinates is not None else (psi[:len(bases)],)*2
         self.coors = Coordinates(*coors)
-        self.hi = self.coors.get_scaling_factors()
+        self.hi = self.coors.hi
+        self.sg = self.coors.sg
         shape = list(self.global_shape())
         self.axes = axes
         assert shape
-        assert min(shape) > 0
+        #assert min(shape) > 0
+        if min(shape) == 0:
+            self.subcomm = comm
+            self._dtype = dtype
+            return
 
         if axes is not None:
             axes = list(axes) if np.ndim(axes) else [axes]
@@ -188,7 +208,7 @@ class TensorProductSpace(PFFT):
             assert sorted(axes[i]) == sorted(set(axes[i]))
 
         if dtype is None:
-            dtype = np.complex if isinstance(bases[axes[-1][-1]], C2CBasis) else np.float
+            dtype = np.complex if isinstance(self.bases[axes[-1][-1]], C2C) else np.float
 
         dtype = np.dtype(dtype)
         assert dtype.char in 'fdgFDG'
@@ -204,9 +224,9 @@ class TensorProductSpace(PFFT):
 
         self.axes = axes
         if not backward_from_pencil:
-            if isinstance(bases[axes[-1][-1]], C2CBasis):
+            if isinstance(self.bases[axes[-1][-1]], C2C):
                 assert np.dtype(dtype).char in 'FDG'
-            elif isinstance(bases[axes[-1][-1]], R2CBasis):
+            elif isinstance(self.bases[axes[-1][-1]], R2C):
                 assert np.dtype(dtype).char in 'fdg'
 
             if isinstance(comm, Subcomm):
@@ -227,10 +247,10 @@ class TensorProductSpace(PFFT):
 
             # At this points axes is a tuple of tuples of length one.
             # Try to collapse some Fourier transforms into one.
-            if np.any([abs(base.padding_factor - 1.0) > 1e-6 for base in bases]):
+            if np.any([abs(base.padding_factor - 1.0) > 1e-6 for base in self.bases]):
                 collapse_fourier = False
             if collapse_fourier:
-                F = lambda ax: bases[ax].family() == 'fourier' and self.subcomm[ax].Get_size() == 1
+                F = lambda ax: self.bases[ax].family() == 'fourier' and self.subcomm[ax].Get_size() == 1
                 axis = self.axes[-1][-1]
                 groups = [list(self.axes[-1])]
                 F0 = F(axis)
@@ -286,15 +306,11 @@ class TensorProductSpace(PFFT):
                 [o.scalar_product for o in self.xfftn],
                 [o.forward for o in self.transfer],
                 self.pencil)
-            self.backward_uniform = Transform(
-                [o.backward_uniform for o in self.xfftn[::-1]],
-                [o.backward for o in self.transfer[::-1]],
-                self.pencil[::-1])
 
         else:
             self.configure_backwards(backward_from_pencil, dtype, kw)
 
-        for i, base in enumerate(bases):
+        for i, base in enumerate(self.bases):
             base.axis = i
             if base.has_nonhomogeneous_bcs:
                 base.bc.set_tensor_bcs(base, self)
@@ -324,9 +340,9 @@ class TensorProductSpace(PFFT):
         self.xfftn.append(xfftn)
         self.subcomm = pencil.subcomm
         subshape = list(pencil.subshape)
-        if isinstance(xfftn, R2CBasis):
+        if isinstance(xfftn, R2C):
             subshape[axes[-1]] = int(np.floor(xfftn.N*xfftn.padding_factor))
-            dtype = np.float
+            dtype = np.dtype(dtype.char.lower())
         else:
             subshape[axes[-1]] = int(np.floor(subshape[axes[-1]]*xfftn.padding_factor))
         self.xfftn[-1].plan(subshape, axes, dtype, kw)
@@ -341,9 +357,9 @@ class TensorProductSpace(PFFT):
             transBA = pencilA.transfer(pencilB, dtype)
             xfftn = self.bases[axes[-1]]
             subshape = list(pencilB.subshape)
-            if isinstance(xfftn, R2CBasis):
+            if isinstance(xfftn, R2C):
                 subshape[axes[-1]] = int(np.floor(xfftn.N*xfftn.padding_factor))
-                dtype = np.float
+                dtype = np.dtype(dtype.char.lower())
             else:
                 subshape[axes[-1]] = int(np.floor(subshape[axes[-1]]*xfftn.padding_factor))
             xfftn.plan(subshape, axes, dtype, kw)
@@ -372,10 +388,6 @@ class TensorProductSpace(PFFT):
             [o.scalar_product for o in self.xfftn[::-1]],
             [o.forward for o in self.transfer[::-1]],
             self.pencil[::-1])
-        self.backward_uniform = Transform(
-            [o.backward_uniform for o in self.xfftn],
-            [o.forward for o in self.transfer],
-            self.pencil)
 
     def get_dealiased(self, padding_factor=1.5, dealias_direct=False):
         if isinstance(padding_factor, Number):
@@ -397,15 +409,21 @@ class TensorProductSpace(PFFT):
 
     def get_refined(self, N):
         if isinstance(N, Number):
-            N = (N,)*len(self)
+            N = N*np.array(self.global_shape())
         elif isinstance(N, (tuple, list, np.ndarray)):
             assert len(N) == len(self)
         refined_bases = [base.get_refined(N[axis])
                          for axis, base in enumerate(self.bases)]
         return TensorProductSpace(self.subcomm, refined_bases, axes=self.axes,
-                                  dtype=self.forward.input_array.dtype,
+                                  dtype=self.dtype(),
                                   coordinates=self.coors.coordinates)
 
+    def dtype(self, forward_output=False):
+        """Return datatype function space is planned for"""
+        if hasattr(self, 'forward'):
+            return PFFT.dtype(self, forward_output)
+        return self._dtype
+
     def convolve(self, a_hat, b_hat, ab_hat):
         """Convolution of a_hat and b_hat
 
@@ -492,7 +510,7 @@ class TensorProductSpace(PFFT):
             x = base.map_reference_domain(points[axis])
             D = base.evaluate_basis_all(x=x, argument=1)
             P = D[..., self.local_slice()[axis]]
-            if isinstance(base, R2CBasis):
+            if isinstance(base, R2C):
                 M = base.N//2+1
                 if base.N % 2 == 0:
                     last_conj_index = M-1
@@ -509,7 +527,7 @@ class TensorProductSpace(PFFT):
 
             if len(out) == 0:
                 out = np.tensordot(P, coefficients, (1, axis))
-                if isinstance(base, R2CBasis):
+                if isinstance(base, R2C):
                     ss = [slice(None)]*len(self)
                     ss[axis] = slice(sl, st)
                     out += np.conj(np.tensordot(P[tuple(sp)], coefficients[tuple(ss)], (1, axis)))
@@ -518,7 +536,7 @@ class TensorProductSpace(PFFT):
             else:
                 k = np.count_nonzero([m < axis for m in previous_axes])
                 if len(self) == 2:
-                    if not isinstance(base, R2CBasis):
+                    if not isinstance(base, R2C):
                         out2 = np.sum(P*out, axis=-1)
                     else:
                         sp = tuple(sp)
@@ -531,7 +549,7 @@ class TensorProductSpace(PFFT):
                     if len(out.shape) == 3:
                         kk = 1 if axis-k == 1 else 2
                         sx[kk] = np.newaxis
-                    if not isinstance(base, R2CBasis):
+                    if not isinstance(base, R2C):
                         out2 = np.sum(P[tuple(sx)]*out, axis=1+axis-k)
                     else:
                         out2 = np.sum(P[tuple(sx)].real*out.real - P[tuple(sx)].imag*out.imag, axis=1+axis-k)
@@ -543,7 +561,7 @@ class TensorProductSpace(PFFT):
                 out = out2
             previous_axes.append(axis)
         output_array[:] = out
-        output_array = self.comm.allreduce(output_array)
+        output_array = comm.allreduce(output_array)
         return output_array
 
 
@@ -565,7 +583,7 @@ class TensorProductSpace(PFFT):
         w = []
         for base in self:
             axis = base.axis
-            if isinstance(base, R2CBasis):
+            if isinstance(base, R2C):
                 r2c = axis
                 M = base.N//2+1
                 if base.N % 2 == 0:
@@ -583,7 +601,7 @@ class TensorProductSpace(PFFT):
             output_array = evaluate.evaluate_lm_3D(list(self.bases), output_array, coefficients, x[0], x[1], x[2], w[0], w[1], w[2], r2c, last_conj_index, sl)
 
         output_array = np.atleast_1d(output_array)
-        output_array = self.comm.allreduce(output_array)
+        output_array = comm.allreduce(output_array)
         return output_array
 
     def _eval_cython(self, points, coefficients, output_array):
@@ -606,7 +624,7 @@ class TensorProductSpace(PFFT):
             x = base.map_reference_domain(points[axis])
             D = base.evaluate_basis_all(x=x)
             P.append(D[..., self.local_slice()[axis]])
-            if isinstance(base, R2CBasis):
+            if isinstance(base, R2C):
                 r2c = axis
                 M = base.N//2+1
                 if base.N % 2 == 0:
@@ -621,7 +639,7 @@ class TensorProductSpace(PFFT):
             output_array = evaluate.evaluate_3D(output_array, coefficients, P, r2c, last_conj_index, sl)
 
         output_array = np.atleast_1d(output_array)
-        output_array = self.comm.allreduce(output_array)
+        output_array = comm.allreduce(output_array)
         return output_array
 
     def wavenumbers(self, scaled=False, eliminate_highest_freq=False):
@@ -704,7 +722,7 @@ class TensorProductSpace(PFFT):
             return [np.broadcast_to(m, self.shape(False)) for m in lm]
         return lm
 
-    def local_curvilinear_mesh(self, uniform=False):
+    def local_cartesian_mesh(self, uniform=False):
         """Return curvilinear mesh
 
         Parameters
@@ -719,7 +737,7 @@ class TensorProductSpace(PFFT):
             xx.append(sp.lambdify(psi, rv)(*X))
         return xx
 
-    def curvilinear_mesh(self, uniform=False):
+    def cartesian_mesh(self, uniform=False):
         """Return curvilinear mesh
 
         Parameters
@@ -746,6 +764,10 @@ class TensorProductSpace(PFFT):
         """Return number of elements in :class:`.TensorProductSpace`"""
         return np.prod(self.shape(forward_output))
 
+    def slice(self):
+        """The slices of dofs for each dimension"""
+        return tuple(base.slice() for base in self.bases)
+
     def global_shape(self, forward_output=False):
         """Return global shape of arrays for TensorProductSpace
 
@@ -757,10 +779,7 @@ class TensorProductSpace(PFFT):
             space, i.e., the input to a forward transform.
 
         """
-        #if not forward_output:
-        #    return tuple([int(np.round(base.shape(forward_output)*base.padding_factor)) for base in self])
         return tuple([base.shape(forward_output) for base in self])
-        #return self.shape(forward_output)
 
     def mask_nyquist(self, u_hat, mask=None):
         """Return Function `u_hat` with zero Nyquist coefficients
@@ -814,7 +833,7 @@ class TensorProductSpace(PFFT):
         ----------
         u : Array
         """
-        dx = self.hi.prod()
+        dx = self.coors.get_sqrt_det_g()
         mesh = self.local_mesh(True)
         if dx == 1:
             return
@@ -836,6 +855,10 @@ class TensorProductSpace(PFFT):
         """Return tensor rank of TensorProductSpace"""
         return 0
 
+    @property
+    def tensor_rank(self):
+        return 0
+
     def __len__(self):
         """Return dimension of TensorProductSpace"""
         return len(self.bases)
@@ -857,6 +880,29 @@ class TensorProductSpace(PFFT):
                 axes.append(axis)
         return axes
 
+    def get_nondiagonal_axes(self):
+        """Return list of axes that may contain non-diagonal matrices"""
+        axes = []
+        if isinstance(self.hi.prod(), sp.Expr):
+            x = 'xyzrs'
+            sym = self.hi.prod().free_symbols
+            msaxes = set()
+            for s in sym:
+                msaxes.add(x.index(str(s)))
+        for axis, base in enumerate(self):
+            if not base.family() == 'fourier':
+                axes.append(axis)
+            if base.family() == 'fourier' and (axis in msaxes):
+                axes.append(axis)
+        return axes
+
+    def get_nonhomogeneous_axes(self):
+        axes = []
+        for axis, base in enumerate(self):
+            if base.has_nonhomogeneous_bcs:
+                axes.append(axis)
+        return axes
+
     @property
     def is_orthogonal(self):
         ortho = True
@@ -872,6 +918,52 @@ class TensorProductSpace(PFFT):
                                   dtype=self.forward.input_array.dtype,
                                   coordinates=self.coors.coordinates)
 
+    def get_adaptive(self, fun=None, reltol=1e-12, abstol=1e-15):
+        """Return space (otherwise as self) with number of quadrature points
+        determined by fitting `fun`
+
+        Returns
+        -------
+        TensorProductSpace
+            A new space with adaptively found number of quadrature points
+
+        Note
+        ----
+        Only spaces defined with N=0 quadrature points are adapted, the
+        others are kept as is
+
+        Example
+        -------
+        >>> from shenfun import FunctionSpace, Function, TensorProductSpace, comm
+        >>> import sympy as sp
+        >>> r, theta = psi = sp.symbols('x,y', real=True, positive=True)
+        >>> rv = (r*sp.cos(theta), r*sp.sin(theta))
+        >>> B0 = FunctionSpace(0, 'C', domain=(0, 1))
+        >>> F0 = FunctionSpace(0, 'F', dtype='d')
+        >>> T = TensorProductSpace(comm, (B0, F0), coordinates=(psi, rv))
+        >>> u = Function(T, buffer=((1-r)*r)**4*(sp.cos(10*theta)))
+        >>> print(u.global_shape)
+        (9, 11)
+
+        """
+        domains = [base.domain for base in self.bases]
+        sym0 = fun.free_symbols
+        sd = {str(sym): sym for sym in sym0}
+        Tj = []
+        for axis, base in enumerate(self.bases):
+            # First remove all other axes from the function
+            otheraxes = list(range(len(self.bases)))
+            otheraxes.remove(axis)
+            fj = fun.copy()
+            for ax in otheraxes:
+                fj = fj.subs(sd['xyzrs'[ax]], 0.5*(domains[ax][1]-domains[ax][0]))
+            if base.N == 0:
+                Tj.append(base.get_adaptive(fun=fj, reltol=reltol, abstol=abstol))
+            else:
+                Tj.append(base)
+        return TensorProductSpace(self.subcomm, Tj, axes=self.axes,
+                                  coordinates=self.coors.coordinates)
+
     @property
     def is_composite_space(self):
         return 0
@@ -908,11 +1000,11 @@ class TensorProductSpace(PFFT):
         return compatible
 
 
-class MixedTensorProductSpace(object):
+class CompositeSpace:
     """Class for composite tensorproductspaces.
 
-    The mixed spaces are Cartesian products of TensorproductSpaces or
-    other MixedTensorProductSpaces.
+    The mixed spaces are Cartesian products of TensorProductSpaces or
+    other CompositeSpaces.
 
     Parameters
     ----------
@@ -924,7 +1016,6 @@ class MixedTensorProductSpace(object):
         self.spaces = spaces
         self.forward = VectorTransform([space.forward for space in spaces])
         self.backward = VectorTransform([space.backward for space in spaces])
-        self.backward_uniform = VectorTransform([space.backward_uniform for space in spaces])
         self.scalar_product = VectorTransform([space.scalar_product for space in spaces])
 
     @property
@@ -991,7 +1082,25 @@ class MixedTensorProductSpace(object):
 
     @property
     def rank(self):
-        """Return rank of space"""
+        """Return rank of space
+
+        Note
+        ----
+        This is 1 for all composite spaces, since we use a flat
+        storage of all composite arrays.
+        """
+        return 1
+
+    @property
+    def tensor_rank(self):
+        """Return rank of tensor
+
+        Note
+        ----
+        This is the number of free indices in the tensor, 0 for scalar,
+        1 for vector etc. It is None for a composite space that is not a
+        tensor.
+        """
         return None
 
     def dim(self):
@@ -1009,7 +1118,7 @@ class MixedTensorProductSpace(object):
         return s
 
     def size(self, forward_output=False):
-        """Return number of elements in :class:`.MixedTensorProductSpace`"""
+        """Return number of elements in :class:`.CompositeSpace`"""
         N = self.shape(forward_output)
         if forward_output:
             return np.sum([np.prod(s) for s in N])
@@ -1017,7 +1126,7 @@ class MixedTensorProductSpace(object):
             return np.prod(N)
 
     def shape(self, forward_output=False):
-        """Return shape of arrays for MixedTensorProductSpace
+        """Return shape of arrays for CompositeSpace
 
         Parameters
         ----------
@@ -1028,7 +1137,7 @@ class MixedTensorProductSpace(object):
 
         Note
         ----
-        A :class:`.MixedTensorProductSpace` may contain tensor product spaces
+        A :class:`.CompositeSpace` may contain tensor product spaces
         of different shape in spectral space. Hence this function returns a
         list of shapes and not one single tuple.
         """
@@ -1042,7 +1151,7 @@ class MixedTensorProductSpace(object):
         return s
 
     def global_shape(self, forward_output=False):
-        """Return global shape for MixedTensorProductSpace
+        """Return global shape for CompositeSpace
 
         Parameters
         ----------
@@ -1068,6 +1177,10 @@ class MixedTensorProductSpace(object):
         s = self.flatten()[0].local_slice(forward_output)
         return (slice(None),) + s
 
+    def slice(self):
+        """The slices of dofs for each dimension"""
+        return tuple(space.slice() for space in self.flatten())
+
     def num_components(self):
         """Return number of spaces in mixed space"""
         f = self.flatten()
@@ -1099,7 +1212,7 @@ class MixedTensorProductSpace(object):
     def get_orthogonal(self):
         raise NotImplementedError
 
-    def get_dealiased(self):
+    def get_dealiased(self, padding_factor=1.5, dealias_direct=False):
         raise NotImplementedError
 
     def __getitem__(self, i):
@@ -1114,13 +1227,16 @@ class MixedTensorProductSpace(object):
         return len(self.spaces)
 
 
-class VectorTensorProductSpace(MixedTensorProductSpace):
-    """A special :class:`.MixedTensorProductSpace` where the number of spaces
-    must equal the geometrical dimension of the problem.
+class VectorSpace(CompositeSpace):
+    """Vector of :class:`.TensorProductSpace`s
+
+    This is simply a special :class:`.CompositeSpace` where the number of
+    :class:`.TensorProductSpace`s equals the geometrical dimension of the
+    problem.
 
     For example, a TensorProductSpace created by a tensorproduct of 2 1D
-    bases, will have vectors of length 2. A TensorProductSpace created from 3
-    1D bases will have vectors of length 3.
+    spaces, will have vectors of length 2. A TensorProductSpace created from 3
+    1D spaces will have vectors of length 3.
 
     Parameters
     ----------
@@ -1135,7 +1251,7 @@ class VectorTensorProductSpace(MixedTensorProductSpace):
             spaces = space
         else:
             spaces = [space]*space.dimensions
-        MixedTensorProductSpace.__init__(self, spaces)
+        CompositeSpace.__init__(self, spaces)
 
     def num_components(self):
         """Return number of spaces in mixed space"""
@@ -1143,12 +1259,11 @@ class VectorTensorProductSpace(MixedTensorProductSpace):
         return self.dimensions
 
     @property
-    def rank(self):
-        """Return tensor rank of space"""
+    def tensor_rank(self):
         return 1
 
     def shape(self, forward_output=False):
-        """Return shape of arrays for VectorTensorProductSpace
+        """Return shape of arrays for VectorSpace
 
         Parameters
         ----------
@@ -1164,20 +1279,50 @@ class VectorTensorProductSpace(MixedTensorProductSpace):
 
     def get_refined(self, N):
         if np.all([s == self.spaces[0] for s in self.spaces[1:]]):
-            return VectorTensorProductSpace(self.spaces[0].get_refined(N))
-        return VectorTensorProductSpace([s.get_refined(N) for s in self.spaces])
+            return VectorSpace(self.spaces[0].get_refined(N))
+        return VectorSpace([s.get_refined(N) for s in self.spaces])
 
     def get_dealiased(self, padding_factor=1.5, dealias_direct=False):
         if np.all([s == self.spaces[0] for s in self.spaces[1:]]):
-            return VectorTensorProductSpace(self.spaces[0].get_dealiased(padding_factor, dealias_direct))
-        return VectorTensorProductSpace([s.get_dealiased(padding_factor, dealias_direct) for s in self.spaces])
+            return VectorSpace(self.spaces[0].get_dealiased(padding_factor, dealias_direct))
+        return VectorSpace([s.get_dealiased(padding_factor, dealias_direct) for s in self.spaces])
 
     def get_orthogonal(self):
         if np.all([s == self.spaces[0] for s in self.spaces[1:]]):
-            return VectorTensorProductSpace(self.spaces[0].get_orthogonal())
-        return VectorTensorProductSpace([s.get_orthogonal() for s in self.spaces])
+            return VectorSpace(self.spaces[0].get_orthogonal())
+        return VectorSpace([s.get_orthogonal() for s in self.spaces])
 
-class VectorTransform(object):
+class TensorSpace(VectorSpace):
+    """A special :class:`.CompositeSpace` for second rank tensors.
+
+    For example, a TensorProductSpace created by a tensorproduct of 2 1D
+    bases, will have tensors of length 4. A TensorProductSpace created from 3
+    1D bases will have tensors of length 9.
+
+    Parameters
+    ----------
+    space : :class:`.TensorProductSpace` or list of ndim :class:`.VectorSpace`s
+        Spaces to create vector from
+
+    """
+
+    def __init__(self, space):
+        if isinstance(space, list):
+            spaces = space
+        else:
+            spaces = [VectorSpace(space)]*space.dimensions
+        CompositeSpace.__init__(self, spaces)
+
+    def num_components(self):
+        """Return number of spaces in mixed space"""
+        return self.dimensions**2
+
+    @property
+    def tensor_rank(self):
+        return 2
+
+
+class VectorTransform:
 
     __slots__ = ('_transforms',)
 
@@ -1202,7 +1347,7 @@ class VectorTransform(object):
         return output_array
 
 
-class Convolve(object):
+class Convolve:
     r"""Class for convolving without truncation.
 
     The convolution of :math:`\hat{a}` and :math:`\hat{b}` is computed by first
@@ -1263,7 +1408,7 @@ class Convolve(object):
         return ab_hat
 
 
-class BoundaryValues(object):
+class BoundaryValues:
     """Class for setting nonhomogeneous boundary conditions for a 1D Dirichlet
     base inside a multidimensional TensorProductSpace.
 
@@ -1288,7 +1433,7 @@ class BoundaryValues(object):
     def update_bcs(self, bc=None):
         if bc is not None:
             assert isinstance(bc, (list, tuple))
-            assert len(bc) in (2, 4)
+            assert len(bc) in (2, 4, 6)
             self.bc = list(bc)
             for i, bci in enumerate(bc):
                 if isinstance(bci, (Number, sp.Expr, np.ndarray)):
@@ -1371,16 +1516,6 @@ class BoundaryValues(object):
                         Yi.append(X[k][this_base.si[j]])
                     f_bci = lbci(*Yi)
 
-                    #x, y, z, tt = sp.symbols("x,y,z,t")
-                    #sym0 = [sym for sym in (x, y, z) if sym in bci.free_symbols]
-                    #if tt in bci.free_symbols:
-                    #    bci = bci.subs({'t': self.bc_time})
-                    #lbci = sp.lambdify(sym0, bci, 'numpy')
-                    #Yi = []
-                    #for i, ax in enumerate((x, y, z)):
-                    #    if ax in bci.free_symbols:
-                    #        Yi.append(X[i][this_base.si[j]])
-                    #f_bci = lbci(*Yi)
                     # Put the Dirichlet value in the position of the bc dofs
                     if s.stop == int(this_base.N*this_base.padding_factor):
                         b[this_base.si[-(len(self.bc))+j]] = f_bci
@@ -1392,51 +1527,80 @@ class BoundaryValues(object):
                 else:
                     raise NotImplementedError
 
-            if number_of_bases_after_this == 0:
-                # Dirichlet base is the first to be transformed
-                b_hat = b
+            if len(T.get_nonhomogeneous_axes()) == 1:
+                if number_of_bases_after_this == 0:
+                    # Dirichlet base is the first to be transformed
+                    b_hat = b
 
-            elif number_of_bases_after_this == 1:
+                elif number_of_bases_after_this == 1:
+                    T.forward._xfftn[0].input_array[...] = b
+
+                    T.forward._xfftn[0]()
+                    arrayA = T.forward._xfftn[0].output_array
+                    arrayB = T.forward._xfftn[1].input_array
+                    T.forward._transfer[0](arrayA, arrayB)
+                    b_hat = arrayB.copy()
+
+                elif number_of_bases_after_this == 2:
+                    T.forward._xfftn[0].input_array[...] = b
+
+                    T.forward._xfftn[0]()
+                    arrayA = T.forward._xfftn[0].output_array
+                    arrayB = T.forward._xfftn[1].input_array
+                    T.forward._transfer[0](arrayA, arrayB)
+
+                    T.forward._xfftn[1]()
+                    arrayA = T.forward._xfftn[1].output_array
+                    arrayB = T.forward._xfftn[2].input_array
+                    T.forward._transfer[1](arrayA, arrayB)
+                    b_hat = arrayB.copy()
+
+                # Now b_hat contains the correct slices in slm1 and slm2
+                # These are the values to use on intermediate steps. If for example the Dirichlet space is squeezed between two Fourier spaces
+                for i in range(len(self.bc)):
+                    self.bcs[i] = b_hat[this_base.si[-(len(self.bc))+i]].copy()
+
+                # Final (the values to set on fully transformed functions)
                 T.forward._xfftn[0].input_array[...] = b
+                for i in range(len(T.forward._transfer)):
 
-                T.forward._xfftn[0]()
-                arrayA = T.forward._xfftn[0].output_array
-                arrayB = T.forward._xfftn[1].input_array
-                T.forward._transfer[0](arrayA, arrayB)
-                b_hat = arrayB.copy()
+                    T.forward._xfftn[i]()
+                    arrayA = T.forward._xfftn[i].output_array
+                    arrayB = T.forward._xfftn[i+1].input_array
+                    T.forward._transfer[i](arrayA, arrayB)
 
-            elif number_of_bases_after_this == 2:
-                T.forward._xfftn[0].input_array[...] = b
+                T.forward._xfftn[-1]()
+                b_hat = T.forward._xfftn[-1].output_array
+                for i in range(len(self.bc)):
+                    self.bcs_final[i] = b_hat[this_base.si[-(len(self.bc))+i]].copy()
 
-                T.forward._xfftn[0]()
-                arrayA = T.forward._xfftn[0].output_array
-                arrayB = T.forward._xfftn[1].input_array
-                T.forward._transfer[0](arrayA, arrayB)
+            else: # 2 non-homogeneous directions
+                assert len(T.bases) == 2, 'Only implemented for 2D'
+                from shenfun import project
+                bases = []
+                for axis, base in enumerate(T.bases):
+                    if not base is this_base:
+                        bases.append(base.get_unplanned())
 
-                T.forward._xfftn[1]()
-                arrayA = T.forward._xfftn[1].output_array
-                arrayB = T.forward._xfftn[2].input_array
-                T.forward._transfer[1](arrayA, arrayB)
-                b_hat = arrayB.copy()
+                other_base = bases[0]
+                ua = Array(other_base)
+                for j, bcj in enumerate(other_base.bc.bc.copy()):
+                    xj = this_base.domain[j]
+                    sym = sp.sympify(bcj).free_symbols
+                    if len(sym) == 1:
+                        s = bcj.subs(sym.pop(), xj)
+                        other_base.bc.bc[j] = s
+                        other_base.bc.bcs[j] = s
+                        other_base.bc.bcs_final[j] = s
+                    else:
+                        other_base.bc.bc[j] = bcj
+                        other_base.bc.bcs[j] = bcj
+                        other_base.bc.bcs_final[j] = bcj
 
-            # Now b_hat contains the correct slices in slm1 and slm2
-            # These are the values to use on intermediate steps. If for example the Dirichlet space is squeezed between two Fourier spaces
-            for i in range(len(self.bc)):
-                self.bcs[i] = b_hat[this_base.si[-(len(self.bc))+i]].copy()
+                for j in range(len(self.bc)):
+                    ua[:] = b[this_base.si[-(len(self.bc))+j]]
+                    self.bcs_final[j] = project(ua, other_base)
 
-            # Final (the values to set on fully transformed functions)
-            T.forward._xfftn[0].input_array[...] = b
-            for i in range(len(T.forward._transfer)):
-
-                T.forward._xfftn[i]()
-                arrayA = T.forward._xfftn[i].output_array
-                arrayB = T.forward._xfftn[i+1].input_array
-                T.forward._transfer[i](arrayA, arrayB)
-
-            T.forward._xfftn[-1]()
-            b_hat = T.forward._xfftn[-1].output_array
-            for i in range(len(self.bc)):
-                self.bcs_final[i] = b_hat[this_base.si[-(len(self.bc))+i]].copy()
 
     def add_to_orthogonal(self, u, uh):
         """Add contribution from boundary functions to `u`
@@ -1451,6 +1615,9 @@ class BoundaryValues(object):
         B = self.base.get_bc_basis()
         M = B.coefficient_matrix().T
         sl = B.slice()
+        ds = uh.shape[self.base.axis] - B.N
+        if ds > 0: # uh is padded, but B is not. Boundary dofs are in padded locations.
+            sl = slice(sl.start + ds, sl.stop + ds)
         if u.ndim > 1:
             sl = self.base.sl[sl]
         for i, row in enumerate(M):
@@ -1465,8 +1632,11 @@ class BoundaryValues(object):
             Add boundary values to this Function
 
         """
+        self.set_boundary_dofs(u)
+        if not self.has_nonhomogeneous_bcs(): # there's no contribution for homogeneous bcs
+            return
         coors = self.tensorproductspace.coors if self.tensorproductspace else self.base.coors
-        M = self.base.get_bcmass_matrix(coors.hi.prod())
+        M = self.base.get_bcmass_matrix(coors.get_sqrt_det_g())
         w0 = np.zeros_like(u)
         u -= M.matvec(u, w0, axis=self.base.axis)
 
@@ -1507,14 +1677,12 @@ class BoundaryValues(object):
 
 def some_basic_tests():
     import pyfftw             #pylint: disable=import-error
-    from mpi4py import MPI
 
-    comm = MPI.COMM_WORLD
     N = 8
-    K0 = C2CBasis(N)
-    K1 = C2CBasis(N)
-    K2 = C2CBasis(N)
-    K3 = R2CBasis(N)
+    K0 = C2C(N)
+    K1 = C2C(N)
+    K2 = C2C(N)
+    K3 = R2C(N)
     T = TensorProductSpace(comm, (K0, K1, K2, K3))
 
     # Create data on rank 0 for testing
@@ -1548,10 +1716,10 @@ def some_basic_tests():
 
     # Padding
     # Needs new instances of bases because arrays have new sizes
-    Kp0 = C2CBasis(N, padding_factor=1.5)
-    Kp1 = C2CBasis(N, padding_factor=1.5)
-    Kp2 = C2CBasis(N, padding_factor=1.5)
-    Kp3 = R2CBasis(N, padding_factor=1.5)
+    Kp0 = C2C(N, padding_factor=1.5)
+    Kp1 = C2C(N, padding_factor=1.5)
+    Kp2 = C2C(N, padding_factor=1.5)
+    Kp3 = R2C(N, padding_factor=1.5)
     Tp = TensorProductSpace(comm, (Kp0, Kp1, Kp2, Kp3))
 
     f_g_pad = Tp.backward(f_hat)
