@@ -13,30 +13,9 @@ import sympy as sp
 from scipy.fftpack import dct
 from shenfun.optimization import optimizer
 
-__all__ = ['inheritdocstrings', 'dx', 'clenshaw_curtis1D', 'CachedArrayDict',
-           'outer', 'apply_mask', 'integrate_sympy']
+__all__ = ['dx', 'clenshaw_curtis1D', 'CachedArrayDict',
+           'outer', 'apply_mask', 'integrate_sympy', 'mayavi_show']
 
-def inheritdocstrings(cls):
-    """Method used for inheriting docstrings from parent class
-
-    Use as decorator::
-
-         @inheritdocstrings
-         class Child(Parent):
-
-    and Child will use the same docstrings as parent even if
-    a method is overloaded. The Child class may overload the
-    docstring as well and a new docstring defined for a method
-    in Child will overload the Parent.
-    """
-    for name, func in vars(cls).items():
-        if isinstance(func, types.FunctionType) and not func.__doc__:
-            for parent in cls.__bases__:
-                parfunc = getattr(parent, name, None)
-                if parfunc and getattr(parfunc, '__doc__', None):
-                    func.__doc__ = parfunc.__doc__
-                    break
-    return cls
 
 def dx(u):
     r"""Compute integral of u over domain
@@ -118,7 +97,7 @@ class CachedArrayDict(MutableMapping):
             value = self._data[newkey]
         except KeyError:
             shape, dtype, _ = newkey
-            value = np.zeros(shape, dtype=np.dtype(dtype, align=True))
+            value = np.empty(shape, dtype=np.dtype(dtype, align=True))
             self._data[newkey] = value
         if fill:
             value.fill(0)
@@ -217,44 +196,55 @@ def integrate_sympy(f, d):
         p = sp.Poly(f, d[0]).integrate()
         return p(d[2]) - p(d[1])
     except sp.PolynomialError:
+        #return sp.Integral(f, d).evalf()
         return sp.integrate(f, d)
 
 def split(measures):
+    #ms = sp.sympify(measures).expand()
+    #ms = ms if isinstance(ms, tuple) else [ms]
+    #result = []
+    #for m in ms:
+    #    if sp.simplify(m) == 0:
+    #        continue
+    #    d = {'coeff': m} if isinstance(m, Number) else sp.separatevars(m, dict=True)
+    #    d = defaultdict(lambda: 1, {str(k): sp.simplify(v) for k, v in d.items()})
+    #    dc = d['coeff']
+    #    d['coeff'] = int(dc) if isinstance(dc, (sp.Integer, int)) else float(dc)
+    #    result.append(d)
+    #return result
+
+    def _split(mss, result):
+        for ms in mss:
+            ms = sp.sympify(ms)
+            if isinstance(ms, sp.Mul):
+                # Multiplication of two or more terms
+                result = _split(ms.args, result)
+                continue
+
+            # Something else with only one symbol
+            sym = ms.free_symbols
+            assert len(sym) <= 1
+            if len(sym) == 1:
+                sym = sym.pop()
+                result[str(sym)] *= ms
+            else:
+                ms = int(ms) if isinstance(ms, sp.Integer) else float(ms)
+                result['coeff'] *= ms
+        return result
+
     ms = sp.sympify(measures).expand()
-    ms = ms if isinstance(ms, tuple) else [ms]
     result = []
-    for m in ms:
-        d = {'coeff': m} if isinstance(m, Number) else sp.separatevars(m, dict=True)
-        d = defaultdict(lambda: 1, {str(k): v for k, v in d.items()})
-        dc = d['coeff']
-        d['coeff'] = int(dc) if isinstance(dc, (sp.Integer, int)) else float(dc)
-        result.append(d)
+    if isinstance(ms, sp.Add):
+        for arg in ms.args:
+            result.append(_split([arg], defaultdict(lambda: 1)))
+    else:
+        result.append(_split([ms], defaultdict(lambda: 1)))
     return result
 
-    #def _split(mss, result):
-    #    for ms in mss:
-    #        ms = sp.sympify(ms)
-    #        if isinstance(ms, sp.Mul):
-    #            # Multiplication of two or more terms
-    #            result = _split(ms.args, result)
-    #            continue
-
-    #        # Something else with only one symbol
-    #        sym = ms.free_symbols
-    #        assert len(sym) <= 1
-    #        if len(sym) == 1:
-    #            sym = sym.pop()
-    #            result[str(sym)] *= ms
-    #        else:
-    #            ms = int(ms) if isinstance(ms, sp.Integer) else float(ms)
-    #            result['coeff'] *= ms
-    #    return result
-
-    #ms = sp.sympify(measures).expand()
-    #result = []
-    #if isinstance(ms, sp.Add):
-    #    for arg in ms.args:
-    #        result.append(_split([arg], defaultdict(lambda: 1)))
-    #else:
-    #    result.append(_split([ms], defaultdict(lambda: 1)))
-    #return result
+def mayavi_show():
+    """
+    Return show function that updates the mayavi figure in the background.
+    """
+    from pyface.api import GUI
+    from mayavi import mlab
+    return mlab.show(GUI().stop_event_loop)
