@@ -5,7 +5,7 @@ comm = MPI.COMM_WORLD
 
 __all__ = ['LagrangianParticles']
 
-class LagrangianParticles(object):
+class LagrangianParticles:
     """Class for tracking Lagrangian particles
 
     Parameters
@@ -40,15 +40,15 @@ if __name__ == '__main__':
 
     N = (40, 40)
     # Should work for any of these bases
-    #F0 = Basis(N[0], 'F', dtype='D', domain=(0., 1.))
-    #F1 = Basis(N[1], 'F', dtype='d', domain=(0., 1.))
-    F0 = Basis(N[0], 'C', bc=(0, 0), domain=(0., 1.))
-    F1 = Basis(N[1], 'C', bc=(0, 0), domain=(0., 1.))
-    #F0 = Basis(N[0], 'C', domain=(0., 1.))
-    #F1 = Basis(N[1], 'C', domain=(0., 1.))
+    #F0 = FunctionSpace(N[0], 'F', dtype='D', domain=(0., 1.))
+    #F1 = FunctionSpace(N[1], 'F', dtype='d', domain=(0., 1.))
+    F0 = FunctionSpace(N[0], 'C', bc=(0, 0), domain=(0., 1.))
+    F1 = FunctionSpace(N[1], 'C', bc=(0, 0), domain=(0., 1.))
+    #F0 = FunctionSpace(N[0], 'C', domain=(0., 1.))
+    #F1 = FunctionSpace(N[1], 'C', domain=(0., 1.))
 
     T = TensorProductSpace(comm, (F0, F1))
-    TV = VectorTensorProductSpace(T)
+    TV = VectorSpace(T)
 
     x, y = sp.symbols("x,y")
     psi = 1./np.pi*sp.sin(np.pi*x)**2*sp.sin(np.pi*y)**2 # Streamfunction
